using System;
using System.Drawing;
using System.Collections;
using System.ComponentModel;
using System.Windows.Forms;
using System.Data;
using System.IO;
using System.Text;
using System.Globalization;
using Microsoft.Win32;
using DalSemi.OneWire.Adapter;
using DalSemi.OneWire;
using System.IO.Ports;

namespace OneWireIO
{
	/// <summary>
	/// Summary description for Form1.
	/// </summary>
   public class OneWireIO : System.Windows.Forms.Form
   {
      private System.Windows.Forms.Button onewireSearchButton;
      private System.Windows.Forms.Button resetMatchROMButton;
      private System.Windows.Forms.Button resetODMatchROMButton;
      private System.Windows.Forms.Button resetSkipROMbutton;
      private System.Windows.Forms.Button resetODSkipROMButton;

      private PortAdapter adapter = null;
      private bool newline=true;
      private Color COMMENT_COLOR = Color.Black;
      private Color RX_COLOR = Color.Red;
      private Color TX_COLOR = Color.Blue;
      private Color ERROR_COLOR = Color.Purple;
      private int address;
      private int length;
      private String selectedAdapterName;
      private byte[] selectedDevice;
      private byte[] memimage;
      private byte[] romimage;
      private int crc8;
      private int crc16;
      private bool use_resume;
      private bool inOverdrive;
      private byte[] ROM_NO;
      private int LastDiscrepancy;
      private int LastFamilyDiscrepancy;
      private bool LastDeviceFlag;
      private int lasttab;
      private bool afterbit, afterbyte;
      private int other_cnt;
      private const byte family_code = 0x2D;
      private String AppTitle = "1-Wire IO Evaluation Program";
      private String AppVersion = "Version 2.7";
      private bool mask_scroll = false;
      private String LastScriptName = "";
      private bool hide_log_mode = false;

      private System.Windows.Forms.StatusBar StatusBar;
      private System.Windows.Forms.StatusBarPanel AdapterStatus;
      private System.Windows.Forms.StatusBarPanel Status;
      private System.Windows.Forms.Button touchByteButton;
      private System.Windows.Forms.RichTextBox oneWireIOTextBox;
      private System.Windows.Forms.NumericUpDown readBytesCount;
      private System.Windows.Forms.Button readBytesButton;
      private System.Windows.Forms.ComboBox bytesTextBox;
      private System.Windows.Forms.MainMenu mainMenu1;
      private System.Windows.Forms.MenuItem menuItem1;
      private System.Windows.Forms.MenuItem menuItem2;
      private IContainer components;
      private System.Windows.Forms.Button resetButton;
      private System.Windows.Forms.Label label1;

      private RegistryKey key;
      private System.Windows.Forms.Button oneBitButton;
      private System.Windows.Forms.Button zeroBitButton;
      private System.Windows.Forms.Button strongPullupButton;
      private System.Windows.Forms.Button setPowerNormalButton;
      private System.Windows.Forms.TabPage tabRaw;
      private System.Windows.Forms.MenuItem menuItem33;
      private System.Windows.Forms.TabPage setupTab;
      private System.Windows.Forms.TextBox textBox1;
      private System.Windows.Forms.TextBox textBox2;
      private System.Windows.Forms.TextBox textBox3;
      private System.Windows.Forms.ComboBox adapterCombo;
      private System.Windows.Forms.TextBox textAdapterPartNum;
      private System.Windows.Forms.ComboBox portCombo;
      private System.Windows.Forms.TextBox textAdapterOpen;
      private System.Windows.Forms.TextBox textBox4;
      private System.Windows.Forms.CheckBox checkResume;
      private System.Windows.Forms.TextBox textBox5;
      private System.Windows.Forms.TextBox textBox6;
      private System.Windows.Forms.CheckBox checkOverdrive;
      private System.Windows.Forms.ComboBox comboOverdrive;
      private System.Windows.Forms.Button buttonOpenAdapter;
      private System.Windows.Forms.ComboBox comboDeviceList;
      private System.Windows.Forms.TextBox textOverdriveDirections;
      private System.Windows.Forms.TextBox textOverdrive;
      private System.Windows.Forms.TextBox textDevice;
      private System.Windows.Forms.CheckBox checkSearchROM;
      private System.Windows.Forms.ComboBox comboSelectMethod;
      private System.Windows.Forms.Button refreshButton;
      private System.Windows.Forms.TabControl tabControl;
      private System.Windows.Forms.CheckBox autoOpenCheck;
      private System.Windows.Forms.GroupBox lowLevelBox;
      private System.Windows.Forms.ComboBox comboSpeed;
      private System.Windows.Forms.Button readBitButton;
      private System.Windows.Forms.Label label13;
      private System.Windows.Forms.TextBox textBoxROM;
      private System.Windows.Forms.Label label19;
      private System.Windows.Forms.Button strongPullupBitButton;
      private System.Windows.Forms.Button searchROMNextButton;
      private System.Windows.Forms.MenuItem menuItem34;
      private System.Windows.Forms.Button button1;
      private System.Windows.Forms.Button button2;
      private System.Windows.Forms.TextBox searchCommandText;
      private System.Windows.Forms.Label label20;
      private System.Windows.Forms.MenuItem menuItem3;
      private System.Windows.Forms.MenuItem menuHelp;
      private System.Windows.Forms.TextBox textOtherDevices;
      private System.Windows.Forms.GroupBox ROMGroup;
      private System.Windows.Forms.GroupBox DeviceGroup;
      private System.Windows.Forms.GroupBox AdapterGroup;
      private System.Windows.Forms.GroupBox KeyGroup;
      private System.Windows.Forms.RichTextBox logKey;
      private Button button3;
      private Button button4;
      private MenuItem menuItemRunScript;
      private MenuItem menuItem4;
      private MenuItem menuItemSaveScript;
      private Button buttonRepeatScript;
      private NumericUpDown repeatScriptCount;
      private CheckBox checkBoxHideLog;
      private Button button12VPulse;
      private OpenFileDialog openFileDialog;
      enum Block
      {
         Page0, Page1, Page2, Page3, ProtectPage0, ProtectPage1, ProtectPage2, ProtectPage3,
         ProtectReg, Factory, ManufacturerID, Reserved88to8F 
      };

      public OneWireIO()
      {
         string val;
         //
         // Required for Windows Form Designer support
         //
         InitializeComponent();

         //
         // TODO: Add any constructor code after InitializeComponent call
         //

         openFileDialog = new OpenFileDialog();
         openFileDialog.CheckFileExists = true;
         openFileDialog.Multiselect = false;
         openFileDialog.Title = "Select Script";

         checkResume.Checked = false;
         comboSelectMethod.SelectedIndex = 0;
         checkOverdrive.Checked = false;
         checkSearchROM.Checked = true;
         autoOpenCheck.Checked = false;
         adapterCombo.SelectedIndex = 0;
         portCombo.SelectedIndex = 1;
         use_resume = false;
         inOverdrive = false;
         lasttab = 0;
         other_cnt = 0;

         // populate the port drop down (only valid for serial ports)
         string[] commports;
         commports = SerialPort.GetPortNames();
         portCombo.Items.Clear();
         for (int i = 0; i < commports.Length; i++)
            portCombo.Items.Add(commports[i]);

         string adapterName = null, adapterPort = null;
         key = Registry.CurrentUser.OpenSubKey("SOFTWARE\\Maxim Integrated Products\\OneWireAPI.NET\\OneWireIO", true);

         if (key != null)
         {
            val = (string)key.GetValue("Adapter", null);
            adapterCombo.SelectedIndex = int.Parse(val, NumberStyles.Integer);
            
            //??val = (string)key.GetValue("Port", null);
            //??portCombo.SelectedIndex = int.Parse(val, NumberStyles.Integer);

            val = (string)key.GetValue("Port", null);
            if (val != null)
            {
               if (portCombo.Items.Contains(val))
                  portCombo.SelectedIndex = portCombo.Items.IndexOf(val);
               else if (portCombo.Items.Count > 0)
                  portCombo.SelectedIndex = 0;
               else
                  portCombo.SelectedIndex = -1;
            }
            else
               portCombo.SelectedIndex = -1;

            val = (string)key.GetValue("AutoOpen", null);
            if (val != null)
               autoOpenCheck.Checked = (val == "1");
            else
               autoOpenCheck.Checked = false;

            val = (string)key.GetValue("Resume", null);
            if (val != null)
               checkResume.Checked = (val == "1");
            else
               checkResume.Checked = true;

            val = (string)key.GetValue("SelectMethod", null);
            if (val != null)
               comboSelectMethod.SelectedIndex = int.Parse(val, NumberStyles.Integer);
            else
               comboSelectMethod.SelectedIndex = 0;

            val = (string)key.GetValue("Overdrive", null);
            if (val != null)
               checkOverdrive.Checked = (val == "1");
            else
               checkOverdrive.Checked = true;

            // update visiblity of overdrive options
            textOverdrive.Visible = checkOverdrive.Checked;
            textOverdriveDirections.Visible = checkOverdrive.Checked;
            comboOverdrive.Visible = checkOverdrive.Checked;

            val = (string)key.GetValue("OverdriveMethod", null);
            if (val != null)
               comboOverdrive.SelectedIndex = int.Parse(val, NumberStyles.Integer);
            else
               comboOverdrive.SelectedIndex = 0;

            val = (string)key.GetValue("Search", null);
            if (val != null)
               checkSearchROM.Checked = (val == "1");
            else
               checkSearchROM.Checked = true;

            // force check search ROM if match rom is selected
            if ((comboSelectMethod.SelectedIndex == 0) || ((comboOverdrive.SelectedIndex == 0) && checkOverdrive.Checked))
            {
               if (!checkSearchROM.Checked)
                  checkSearchROM.Checked = true;
            }
            // update visiblity of search options
            comboDeviceList.Visible = checkSearchROM.Checked;
            textDevice.Visible = checkSearchROM.Checked;
            refreshButton.Visible = checkSearchROM.Checked;
            textOtherDevices.Text = "";
            textOtherDevices.Visible = checkSearchROM.Checked;
                        
            adapterName = (string)key.GetValue("adapterName", null);
            adapterPort = (string)key.GetValue("adapterPort", null);
            string mru;
            for(int i = 0; (mru=(string)key.GetValue("MRU" + i)) != null; i++)
               bytesTextBox.Items.Add(mru);
         }

         try
         {
            if(adapterName!=null && adapterPort!=null)
               setAdapter(adapterName, adapterPort);//OneWireAccessProvider.getAdapter(adapterName, adapterPort);
         }
         catch(Exception ex)
         {
            addText("Failure: " + ex.Message + "\n" + ex.StackTrace + "\n" + ((ex.InnerException!=null)?ex.InnerException.Message:""), Color.Red);
            AdapterStatus.Text = "Not Loaded"; 
            adapter = null;
         }

         // Read last state from registery
         memimage = new byte[256];
         romimage = new byte[8];
         ROM_NO = new byte[8];

         statusWriteLN("//------------------------------------------------------------------------------------", COMMENT_COLOR);
         statusWriteLN("// " + AppTitle + " - " + AppVersion, COMMENT_COLOR);
         // check for auto open
         bool auto_open_result = false;
         if (autoOpenCheck.Checked)
         {
            // make sure correct port numbers oare indicated
            if (adapterCombo.SelectedIndex == 0)
            {
               textAdapterPartNum.Text = "DS9490R/B";
               portCombo.Items.Clear();
               for (int i = 0; i < 16; i++)
                  portCombo.Items.Add("USB" + i.ToString());
               portCombo.SelectedIndex = 1;
               selectedAdapterName = "{DS9490}";
            }

            auto_open_result = OpenAdapterPort();
         }

         // RP/RN - Reset+Presence/No-presence  <SP_ON>/<SP_OFF> - strong pull-on/off
         // HH - write to device   [HH] - read from device    B - write bit   [B] - read bit
         // <<>> - Error  // line comment  <STD>/<OVR> - standard/overdrive speed
         logKey.SelectionColor = TX_COLOR;
         logKey.AppendText("<STD>");
         logKey.SelectionColor = COMMENT_COLOR;
         logKey.AppendText("    - standard communication speed" + System.Environment.NewLine);
         logKey.SelectionColor = TX_COLOR;
         logKey.AppendText("<OVR>");
         logKey.SelectionColor = COMMENT_COLOR;
         logKey.AppendText("    - overdrive communication speed" + System.Environment.NewLine);
         logKey.SelectionColor = TX_COLOR;
         logKey.AppendText("R");
         logKey.SelectionColor = RX_COLOR;
         logKey.AppendText("P");
         logKey.SelectionColor = COMMENT_COLOR;
         logKey.AppendText("       - Reset and presence pulse" + System.Environment.NewLine);
         logKey.SelectionColor = TX_COLOR;
         logKey.AppendText("R");
         logKey.SelectionColor = RX_COLOR;
         logKey.AppendText("N");
         logKey.SelectionColor = COMMENT_COLOR;
         logKey.AppendText("       - Reset and no presence pulse" + System.Environment.NewLine);
         logKey.SelectionColor = TX_COLOR;
         logKey.AppendText("HH");
         logKey.SelectionColor = COMMENT_COLOR;
         logKey.AppendText("       - write byte to device in hex format" + System.Environment.NewLine);
         logKey.SelectionColor = RX_COLOR;
         logKey.AppendText("[HH]");
         logKey.SelectionColor = COMMENT_COLOR;
         logKey.AppendText("     - read byte from device in hex format" + System.Environment.NewLine);
         logKey.SelectionColor = TX_COLOR;
         logKey.AppendText("B");
         logKey.SelectionColor = COMMENT_COLOR;
         logKey.AppendText("        - write bit to device in binary format" + System.Environment.NewLine);
         logKey.SelectionColor = RX_COLOR;
         logKey.AppendText("[B]");
         logKey.SelectionColor = COMMENT_COLOR;
         logKey.AppendText("      - read bit from device in binary format" + System.Environment.NewLine);          
         logKey.AppendText("//       - line comment " + System.Environment.NewLine);  
         logKey.SelectionColor = TX_COLOR;
         logKey.AppendText("<SP_ON>");
         logKey.SelectionColor = COMMENT_COLOR;
         logKey.AppendText("  - strong pull-up ON" + System.Environment.NewLine);
         logKey.SelectionColor = TX_COLOR;
         logKey.AppendText("<SP_OFF>");
         logKey.SelectionColor = COMMENT_COLOR;
         logKey.AppendText(" - strong pull-up OFF" + System.Environment.NewLine);
         logKey.SelectionColor = ERROR_COLOR;
         logKey.AppendText("<<>>");
         logKey.SelectionColor = COMMENT_COLOR;
         logKey.AppendText("     - error message" + System.Environment.NewLine);


         Text = AppTitle;

         comboSpeed.SelectedIndex = 0;

         // automatically go to the second tab if auto open was a success
         if (auto_open_result)
            tabControl.SelectedIndex = 1;
      }

      /// <summary>
      /// Clean up any resources being used.
      /// </summary>
      protected override void Dispose( bool disposing )
      {
         int val;

         if(key==null)
         {
            key = Registry.CurrentUser.CreateSubKey("SOFTWARE\\Maxim Integrated Products\\OneWireAPI.NET\\OneWireIO");
         }

         try
         {
            key.SetValue("Adapter",adapterCombo.SelectedIndex.ToString());
            //?????key.SetValue("Port",portCombo.SelectedIndex.ToString());
            key.SetValue("Port", portCombo.Items[portCombo.SelectedIndex]);

            val = (autoOpenCheck.Checked) ? 1 : 0;
            key.SetValue("AutoOpen",val.ToString());

            val = (checkResume.Checked) ? 1 : 0;
            key.SetValue("Resume",val.ToString());
                  
            key.SetValue("SelectMethod",comboSelectMethod.SelectedIndex.ToString());
                  
            val = (checkOverdrive.Checked) ? 1 : 0;
            key.SetValue("Overdrive",val.ToString());
                                    
            key.SetValue("OverdriveMethod",comboOverdrive.SelectedIndex.ToString());
                        
            val = (checkSearchROM.Checked) ? 1 : 0;
            key.SetValue("Search",val.ToString());
                        
            key.SetValue("Device",comboDeviceList.SelectedIndex.ToString());

            for(int i=0; i<bytesTextBox.Items.Count; i++)
               key.SetValue("MRU"+i, bytesTextBox.Items[i].ToString());
            
            key.Close();
         }
         catch
         {}
   
         if(adapter!=null)
         {
            try
            {
               adapter.EndExclusive();
               adapter.Dispose();
               adapter = null;
            }
            catch
            {}
         }

         if( disposing )
         {
            if (components != null) 
            {
               components.Dispose();
            }
         }
         base.Dispose( disposing );
      }

      #region Windows Form Designer generated code
      /// <summary>
      /// Required method for Designer support - do not modify
      /// the contents of this method with the code editor.
      /// </summary>
      private void InitializeComponent()
      {
         this.components = new System.ComponentModel.Container();
         System.ComponentModel.ComponentResourceManager resources = new System.ComponentModel.ComponentResourceManager(typeof(OneWireIO));
         this.onewireSearchButton = new System.Windows.Forms.Button();
         this.resetMatchROMButton = new System.Windows.Forms.Button();
         this.resetODMatchROMButton = new System.Windows.Forms.Button();
         this.resetSkipROMbutton = new System.Windows.Forms.Button();
         this.resetODSkipROMButton = new System.Windows.Forms.Button();
         this.StatusBar = new System.Windows.Forms.StatusBar();
         this.AdapterStatus = new System.Windows.Forms.StatusBarPanel();
         this.Status = new System.Windows.Forms.StatusBarPanel();
         this.touchByteButton = new System.Windows.Forms.Button();
         this.oneWireIOTextBox = new System.Windows.Forms.RichTextBox();
         this.readBytesCount = new System.Windows.Forms.NumericUpDown();
         this.readBytesButton = new System.Windows.Forms.Button();
         this.bytesTextBox = new System.Windows.Forms.ComboBox();
         this.mainMenu1 = new System.Windows.Forms.MainMenu(this.components);
         this.menuItem1 = new System.Windows.Forms.MenuItem();
         this.menuItem34 = new System.Windows.Forms.MenuItem();
         this.menuItem33 = new System.Windows.Forms.MenuItem();
         this.menuItem3 = new System.Windows.Forms.MenuItem();
         this.menuItem2 = new System.Windows.Forms.MenuItem();
         this.menuItem4 = new System.Windows.Forms.MenuItem();
         this.menuItemRunScript = new System.Windows.Forms.MenuItem();
         this.menuItemSaveScript = new System.Windows.Forms.MenuItem();
         this.menuHelp = new System.Windows.Forms.MenuItem();
         this.resetButton = new System.Windows.Forms.Button();
         this.label1 = new System.Windows.Forms.Label();
         this.oneBitButton = new System.Windows.Forms.Button();
         this.zeroBitButton = new System.Windows.Forms.Button();
         this.ROMGroup = new System.Windows.Forms.GroupBox();
         this.label20 = new System.Windows.Forms.Label();
         this.searchCommandText = new System.Windows.Forms.TextBox();
         this.button2 = new System.Windows.Forms.Button();
         this.button1 = new System.Windows.Forms.Button();
         this.searchROMNextButton = new System.Windows.Forms.Button();
         this.label19 = new System.Windows.Forms.Label();
         this.textBoxROM = new System.Windows.Forms.TextBox();
         this.strongPullupBitButton = new System.Windows.Forms.Button();
         this.setPowerNormalButton = new System.Windows.Forms.Button();
         this.strongPullupButton = new System.Windows.Forms.Button();
         this.tabControl = new System.Windows.Forms.TabControl();
         this.setupTab = new System.Windows.Forms.TabPage();
         this.KeyGroup = new System.Windows.Forms.GroupBox();
         this.logKey = new System.Windows.Forms.RichTextBox();
         this.DeviceGroup = new System.Windows.Forms.GroupBox();
         this.textOtherDevices = new System.Windows.Forms.TextBox();
         this.refreshButton = new System.Windows.Forms.Button();
         this.textDevice = new System.Windows.Forms.TextBox();
         this.comboDeviceList = new System.Windows.Forms.ComboBox();
         this.checkSearchROM = new System.Windows.Forms.CheckBox();
         this.textOverdriveDirections = new System.Windows.Forms.TextBox();
         this.textOverdrive = new System.Windows.Forms.TextBox();
         this.comboOverdrive = new System.Windows.Forms.ComboBox();
         this.checkOverdrive = new System.Windows.Forms.CheckBox();
         this.textBox6 = new System.Windows.Forms.TextBox();
         this.textBox5 = new System.Windows.Forms.TextBox();
         this.comboSelectMethod = new System.Windows.Forms.ComboBox();
         this.checkResume = new System.Windows.Forms.CheckBox();
         this.AdapterGroup = new System.Windows.Forms.GroupBox();
         this.autoOpenCheck = new System.Windows.Forms.CheckBox();
         this.textBox4 = new System.Windows.Forms.TextBox();
         this.textAdapterOpen = new System.Windows.Forms.TextBox();
         this.buttonOpenAdapter = new System.Windows.Forms.Button();
         this.portCombo = new System.Windows.Forms.ComboBox();
         this.textAdapterPartNum = new System.Windows.Forms.TextBox();
         this.textBox3 = new System.Windows.Forms.TextBox();
         this.textBox2 = new System.Windows.Forms.TextBox();
         this.textBox1 = new System.Windows.Forms.TextBox();
         this.adapterCombo = new System.Windows.Forms.ComboBox();
         this.tabRaw = new System.Windows.Forms.TabPage();
         this.lowLevelBox = new System.Windows.Forms.GroupBox();
         this.checkBoxHideLog = new System.Windows.Forms.CheckBox();
         this.repeatScriptCount = new System.Windows.Forms.NumericUpDown();
         this.buttonRepeatScript = new System.Windows.Forms.Button();
         this.button4 = new System.Windows.Forms.Button();
         this.button3 = new System.Windows.Forms.Button();
         this.label13 = new System.Windows.Forms.Label();
         this.readBitButton = new System.Windows.Forms.Button();
         this.comboSpeed = new System.Windows.Forms.ComboBox();
         this.button12VPulse = new System.Windows.Forms.Button();
         ((System.ComponentModel.ISupportInitialize)(this.AdapterStatus)).BeginInit();
         ((System.ComponentModel.ISupportInitialize)(this.Status)).BeginInit();
         ((System.ComponentModel.ISupportInitialize)(this.readBytesCount)).BeginInit();
         this.ROMGroup.SuspendLayout();
         this.tabControl.SuspendLayout();
         this.setupTab.SuspendLayout();
         this.KeyGroup.SuspendLayout();
         this.DeviceGroup.SuspendLayout();
         this.AdapterGroup.SuspendLayout();
         this.tabRaw.SuspendLayout();
         this.lowLevelBox.SuspendLayout();
         ((System.ComponentModel.ISupportInitialize)(this.repeatScriptCount)).BeginInit();
         this.SuspendLayout();
         // 
         // onewireSearchButton
         // 
         this.onewireSearchButton.Location = new System.Drawing.Point(8, 48);
         this.onewireSearchButton.Name = "onewireSearchButton";
         this.onewireSearchButton.Size = new System.Drawing.Size(152, 24);
         this.onewireSearchButton.TabIndex = 0;
         this.onewireSearchButton.Text = "Reset - Search-ROM (first)";
         this.onewireSearchButton.Click += new System.EventHandler(this.onewireSearchButton_Click);
         // 
         // resetMatchROMButton
         // 
         this.resetMatchROMButton.Location = new System.Drawing.Point(8, 112);
         this.resetMatchROMButton.Name = "resetMatchROMButton";
         this.resetMatchROMButton.Size = new System.Drawing.Size(136, 24);
         this.resetMatchROMButton.TabIndex = 4;
         this.resetMatchROMButton.Text = "Reset - Match ROM";
         this.resetMatchROMButton.Click += new System.EventHandler(this.resetMatchROMButton_Click);
         // 
         // resetODMatchROMButton
         // 
         this.resetODMatchROMButton.Location = new System.Drawing.Point(152, 112);
         this.resetODMatchROMButton.Name = "resetODMatchROMButton";
         this.resetODMatchROMButton.Size = new System.Drawing.Size(136, 24);
         this.resetODMatchROMButton.TabIndex = 5;
         this.resetODMatchROMButton.Text = "Reset - OD Match ROM";
         this.resetODMatchROMButton.Click += new System.EventHandler(this.resetODMatchROMButton_Click);
         // 
         // resetSkipROMbutton
         // 
         this.resetSkipROMbutton.Location = new System.Drawing.Point(8, 80);
         this.resetSkipROMbutton.Name = "resetSkipROMbutton";
         this.resetSkipROMbutton.Size = new System.Drawing.Size(136, 24);
         this.resetSkipROMbutton.TabIndex = 6;
         this.resetSkipROMbutton.Text = "Reset - Skip ROM";
         this.resetSkipROMbutton.Click += new System.EventHandler(this.resetSkipROMbutton_Click);
         // 
         // resetODSkipROMButton
         // 
         this.resetODSkipROMButton.Location = new System.Drawing.Point(152, 80);
         this.resetODSkipROMButton.Name = "resetODSkipROMButton";
         this.resetODSkipROMButton.Size = new System.Drawing.Size(136, 24);
         this.resetODSkipROMButton.TabIndex = 7;
         this.resetODSkipROMButton.Text = "Reset - OD Skip ROM";
         this.resetODSkipROMButton.Click += new System.EventHandler(this.resetODSkipROMButton_Click);
         // 
         // StatusBar
         // 
         this.StatusBar.Location = new System.Drawing.Point(0, 571);
         this.StatusBar.Name = "StatusBar";
         this.StatusBar.Panels.AddRange(new System.Windows.Forms.StatusBarPanel[] {
            this.AdapterStatus,
            this.Status});
         this.StatusBar.ShowPanels = true;
         this.StatusBar.Size = new System.Drawing.Size(512, 22);
         this.StatusBar.TabIndex = 7;
         this.StatusBar.Text = "Normal";
         // 
         // AdapterStatus
         // 
         this.AdapterStatus.Name = "AdapterStatus";
         this.AdapterStatus.Text = "Adapter Not Loaded";
         this.AdapterStatus.Width = 200;
         // 
         // Status
         // 
         this.Status.AutoSize = System.Windows.Forms.StatusBarPanelAutoSize.Spring;
         this.Status.Name = "Status";
         this.Status.Text = "Normal";
         this.Status.Width = 295;
         // 
         // touchByteButton
         // 
         this.touchByteButton.Location = new System.Drawing.Point(8, 80);
         this.touchByteButton.Name = "touchByteButton";
         this.touchByteButton.Size = new System.Drawing.Size(88, 24);
         this.touchByteButton.TabIndex = 9;
         this.touchByteButton.Text = "Write Bytes";
         this.touchByteButton.Click += new System.EventHandler(this.touchByteButton_Click);
         // 
         // oneWireIOTextBox
         // 
         this.oneWireIOTextBox.Anchor = ((System.Windows.Forms.AnchorStyles)((((System.Windows.Forms.AnchorStyles.Top | System.Windows.Forms.AnchorStyles.Bottom)
                     | System.Windows.Forms.AnchorStyles.Left)
                     | System.Windows.Forms.AnchorStyles.Right)));
         this.oneWireIOTextBox.Font = new System.Drawing.Font("Courier New", 9F);
         this.oneWireIOTextBox.Location = new System.Drawing.Point(8, 393);
         this.oneWireIOTextBox.Name = "oneWireIOTextBox";
         this.oneWireIOTextBox.Size = new System.Drawing.Size(472, 143);
         this.oneWireIOTextBox.TabIndex = 0;
         this.oneWireIOTextBox.TabStop = false;
         this.oneWireIOTextBox.Text = "";
         this.oneWireIOTextBox.WordWrap = false;
         // 
         // readBytesCount
         // 
         this.readBytesCount.Font = new System.Drawing.Font("Courier New", 9F);
         this.readBytesCount.Location = new System.Drawing.Point(112, 48);
         this.readBytesCount.Maximum = new decimal(new int[] {
            256,
            0,
            0,
            0});
         this.readBytesCount.Name = "readBytesCount";
         this.readBytesCount.Size = new System.Drawing.Size(48, 21);
         this.readBytesCount.TabIndex = 12;
         this.readBytesCount.TextAlign = System.Windows.Forms.HorizontalAlignment.Center;
         this.readBytesCount.Value = new decimal(new int[] {
            8,
            0,
            0,
            0});
         // 
         // readBytesButton
         // 
         this.readBytesButton.Location = new System.Drawing.Point(8, 48);
         this.readBytesButton.Name = "readBytesButton";
         this.readBytesButton.Size = new System.Drawing.Size(88, 24);
         this.readBytesButton.TabIndex = 13;
         this.readBytesButton.Text = "Read Bytes";
         this.readBytesButton.Click += new System.EventHandler(this.readBytesButton_Click);
         // 
         // bytesTextBox
         // 
         this.bytesTextBox.Font = new System.Drawing.Font("Courier New", 9F);
         this.bytesTextBox.Location = new System.Drawing.Point(112, 80);
         this.bytesTextBox.MaxDropDownItems = 10;
         this.bytesTextBox.Name = "bytesTextBox";
         this.bytesTextBox.Size = new System.Drawing.Size(336, 23);
         this.bytesTextBox.TabIndex = 8;
         this.bytesTextBox.Text = "33 FF FF FF FF FF FF FF FF";
         this.bytesTextBox.DoubleClick += new System.EventHandler(this.bytesTextBox_DoubleClick);
         this.bytesTextBox.Click += new System.EventHandler(this.bytesTextBox_Click);
         // 
         // mainMenu1
         // 
         this.mainMenu1.MenuItems.AddRange(new System.Windows.Forms.MenuItem[] {
            this.menuItem1,
            this.menuItem4,
            this.menuHelp});
         // 
         // menuItem1
         // 
         this.menuItem1.Index = 0;
         this.menuItem1.MenuItems.AddRange(new System.Windows.Forms.MenuItem[] {
            this.menuItem34,
            this.menuItem33,
            this.menuItem3,
            this.menuItem2});
         this.menuItem1.Text = "File";
         // 
         // menuItem34
         // 
         this.menuItem34.Index = 0;
         this.menuItem34.Text = "Clear Log";
         this.menuItem34.Click += new System.EventHandler(this.menuItem34_Click);
         // 
         // menuItem33
         // 
         this.menuItem33.Index = 1;
         this.menuItem33.Text = "Copy Log to Clipboard";
         this.menuItem33.Click += new System.EventHandler(this.menuItem33_Click);
         // 
         // menuItem3
         // 
         this.menuItem3.Index = 2;
         this.menuItem3.Text = "-";
         // 
         // menuItem2
         // 
         this.menuItem2.Index = 3;
         this.menuItem2.Text = "Exit";
         this.menuItem2.Click += new System.EventHandler(this.menuItem2_Click);
         // 
         // menuItem4
         // 
         this.menuItem4.Index = 1;
         this.menuItem4.MenuItems.AddRange(new System.Windows.Forms.MenuItem[] {
            this.menuItemRunScript,
            this.menuItemSaveScript});
         this.menuItem4.Text = "Script";
         // 
         // menuItemRunScript
         // 
         this.menuItemRunScript.Index = 0;
         this.menuItemRunScript.Text = "Run Script";
         this.menuItemRunScript.Click += new System.EventHandler(this.menuItemRunScript_Click);
         // 
         // menuItemSaveScript
         // 
         this.menuItemSaveScript.Index = 1;
         this.menuItemSaveScript.Text = "Save Script";
         this.menuItemSaveScript.Click += new System.EventHandler(this.menuItemSaveScript_Click);
         // 
         // menuHelp
         // 
         this.menuHelp.Index = 2;
         this.menuHelp.Text = "Help";
         this.menuHelp.Click += new System.EventHandler(this.menuHelp_Click);
         // 
         // resetButton
         // 
         this.resetButton.Location = new System.Drawing.Point(8, 16);
         this.resetButton.Name = "resetButton";
         this.resetButton.Size = new System.Drawing.Size(88, 24);
         this.resetButton.TabIndex = 2;
         this.resetButton.Text = "1-Wire Reset";
         this.resetButton.Click += new System.EventHandler(this.resetButton_Click);
         // 
         // label1
         // 
         this.label1.Location = new System.Drawing.Point(168, 48);
         this.label1.Name = "label1";
         this.label1.Size = new System.Drawing.Size(80, 24);
         this.label1.TabIndex = 18;
         this.label1.Text = "Number of Bytes to Read";
         this.label1.TextAlign = System.Drawing.ContentAlignment.MiddleLeft;
         this.label1.Click += new System.EventHandler(this.label1_Click);
         // 
         // oneBitButton
         // 
         this.oneBitButton.Location = new System.Drawing.Point(256, 112);
         this.oneBitButton.Name = "oneBitButton";
         this.oneBitButton.Size = new System.Drawing.Size(88, 24);
         this.oneBitButton.TabIndex = 19;
         this.oneBitButton.Text = "Write 1 Bit";
         this.oneBitButton.Click += new System.EventHandler(this.oneBitButton_Click);
         // 
         // zeroBitButton
         // 
         this.zeroBitButton.Location = new System.Drawing.Point(360, 112);
         this.zeroBitButton.Name = "zeroBitButton";
         this.zeroBitButton.Size = new System.Drawing.Size(88, 24);
         this.zeroBitButton.TabIndex = 20;
         this.zeroBitButton.Text = "Write 0 Bit";
         this.zeroBitButton.Click += new System.EventHandler(this.zeroBitButton_Click);
         // 
         // ROMGroup
         // 
         this.ROMGroup.Controls.Add(this.label20);
         this.ROMGroup.Controls.Add(this.searchCommandText);
         this.ROMGroup.Controls.Add(this.button2);
         this.ROMGroup.Controls.Add(this.button1);
         this.ROMGroup.Controls.Add(this.searchROMNextButton);
         this.ROMGroup.Controls.Add(this.label19);
         this.ROMGroup.Controls.Add(this.textBoxROM);
         this.ROMGroup.Controls.Add(this.onewireSearchButton);
         this.ROMGroup.Controls.Add(this.resetODSkipROMButton);
         this.ROMGroup.Controls.Add(this.resetSkipROMbutton);
         this.ROMGroup.Controls.Add(this.resetODMatchROMButton);
         this.ROMGroup.Controls.Add(this.resetMatchROMButton);
         this.ROMGroup.Location = new System.Drawing.Point(8, 241);
         this.ROMGroup.Name = "ROMGroup";
         this.ROMGroup.Size = new System.Drawing.Size(472, 146);
         this.ROMGroup.TabIndex = 22;
         this.ROMGroup.TabStop = false;
         this.ROMGroup.Text = "ROM Level";
         // 
         // label20
         // 
         this.label20.Location = new System.Drawing.Point(328, 48);
         this.label20.Name = "label20";
         this.label20.Size = new System.Drawing.Size(56, 24);
         this.label20.TabIndex = 27;
         this.label20.Text = "Search Command";
         this.label20.TextAlign = System.Drawing.ContentAlignment.MiddleLeft;
         // 
         // searchCommandText
         // 
         this.searchCommandText.Location = new System.Drawing.Point(392, 48);
         this.searchCommandText.Name = "searchCommandText";
         this.searchCommandText.Size = new System.Drawing.Size(32, 20);
         this.searchCommandText.TabIndex = 26;
         this.searchCommandText.Text = "F0";
         // 
         // button2
         // 
         this.button2.Location = new System.Drawing.Point(296, 80);
         this.button2.Name = "button2";
         this.button2.Size = new System.Drawing.Size(136, 24);
         this.button2.TabIndex = 25;
         this.button2.Text = "Reset - Resume";
         this.button2.Click += new System.EventHandler(this.button2_Click);
         // 
         // button1
         // 
         this.button1.Location = new System.Drawing.Point(296, 112);
         this.button1.Name = "button1";
         this.button1.Size = new System.Drawing.Size(136, 24);
         this.button1.TabIndex = 24;
         this.button1.Text = "Reset - Read ROM";
         this.button1.Click += new System.EventHandler(this.button1_Click_2);
         // 
         // searchROMNextButton
         // 
         this.searchROMNextButton.Location = new System.Drawing.Point(168, 48);
         this.searchROMNextButton.Name = "searchROMNextButton";
         this.searchROMNextButton.Size = new System.Drawing.Size(152, 24);
         this.searchROMNextButton.TabIndex = 23;
         this.searchROMNextButton.Text = "Reset - Search-ROM (next)";
         this.searchROMNextButton.Click += new System.EventHandler(this.button1_Click_1);
         // 
         // label19
         // 
         this.label19.Location = new System.Drawing.Point(8, 16);
         this.label19.Name = "label19";
         this.label19.Size = new System.Drawing.Size(240, 24);
         this.label19.TabIndex = 22;
         this.label19.Text = "ROM to use with ROM commands                  (auto filled with Search-ROM and Re" +
             "ad-ROM)";
         this.label19.TextAlign = System.Drawing.ContentAlignment.MiddleLeft;
         // 
         // textBoxROM
         // 
         this.textBoxROM.Location = new System.Drawing.Point(304, 16);
         this.textBoxROM.Name = "textBoxROM";
         this.textBoxROM.Size = new System.Drawing.Size(144, 20);
         this.textBoxROM.TabIndex = 12;
         this.textBoxROM.DoubleClick += new System.EventHandler(this.textBoxROM_DoubleClick);
         // 
         // strongPullupBitButton
         // 
         this.strongPullupBitButton.Location = new System.Drawing.Point(199, 147);
         this.strongPullupBitButton.Name = "strongPullupBitButton";
         this.strongPullupBitButton.Size = new System.Drawing.Size(136, 40);
         this.strongPullupBitButton.TabIndex = 3;
         this.strongPullupBitButton.Text = "Start Strong-Pullup after next Bit";
         this.strongPullupBitButton.Click += new System.EventHandler(this.strongPullupBitButton_Click);
         // 
         // setPowerNormalButton
         // 
         this.setPowerNormalButton.Location = new System.Drawing.Point(343, 147);
         this.setPowerNormalButton.Name = "setPowerNormalButton";
         this.setPowerNormalButton.Size = new System.Drawing.Size(104, 40);
         this.setPowerNormalButton.TabIndex = 2;
         this.setPowerNormalButton.Text = "Set Power Normal";
         this.setPowerNormalButton.Click += new System.EventHandler(this.setPowerNormalButton_Click);
         // 
         // strongPullupButton
         // 
         this.strongPullupButton.Location = new System.Drawing.Point(55, 147);
         this.strongPullupButton.Name = "strongPullupButton";
         this.strongPullupButton.Size = new System.Drawing.Size(136, 40);
         this.strongPullupButton.TabIndex = 1;
         this.strongPullupButton.Text = "Start Strong-Pullup after next Byte";
         this.strongPullupButton.Click += new System.EventHandler(this.strongPullupButton_Click);
         // 
         // tabControl
         // 
         this.tabControl.Controls.Add(this.setupTab);
         this.tabControl.Controls.Add(this.tabRaw);
         this.tabControl.Location = new System.Drawing.Point(8, 0);
         this.tabControl.Name = "tabControl";
         this.tabControl.SelectedIndex = 0;
         this.tabControl.Size = new System.Drawing.Size(496, 568);
         this.tabControl.TabIndex = 26;
         this.tabControl.SelectedIndexChanged += new System.EventHandler(this.tabControl1_SelectedIndexChanged);
         // 
         // setupTab
         // 
         this.setupTab.Controls.Add(this.KeyGroup);
         this.setupTab.Controls.Add(this.DeviceGroup);
         this.setupTab.Controls.Add(this.AdapterGroup);
         this.setupTab.Location = new System.Drawing.Point(4, 22);
         this.setupTab.Name = "setupTab";
         this.setupTab.Size = new System.Drawing.Size(488, 542);
         this.setupTab.TabIndex = 2;
         this.setupTab.Text = "Setup";
         // 
         // KeyGroup
         // 
         this.KeyGroup.Controls.Add(this.logKey);
         this.KeyGroup.Location = new System.Drawing.Point(8, 152);
         this.KeyGroup.Name = "KeyGroup";
         this.KeyGroup.Size = new System.Drawing.Size(472, 240);
         this.KeyGroup.TabIndex = 6;
         this.KeyGroup.TabStop = false;
         this.KeyGroup.Text = "1-Wire Log Key";
         // 
         // logKey
         // 
         this.logKey.Anchor = ((System.Windows.Forms.AnchorStyles)((((System.Windows.Forms.AnchorStyles.Top | System.Windows.Forms.AnchorStyles.Bottom)
                     | System.Windows.Forms.AnchorStyles.Left)
                     | System.Windows.Forms.AnchorStyles.Right)));
         this.logKey.Font = new System.Drawing.Font("Courier New", 9F);
         this.logKey.Location = new System.Drawing.Point(16, 24);
         this.logKey.Name = "logKey";
         this.logKey.ReadOnly = true;
         this.logKey.Size = new System.Drawing.Size(432, 200);
         this.logKey.TabIndex = 2;
         this.logKey.TabStop = false;
         this.logKey.Text = "";
         this.logKey.WordWrap = false;
         // 
         // DeviceGroup
         // 
         this.DeviceGroup.Controls.Add(this.textOtherDevices);
         this.DeviceGroup.Controls.Add(this.refreshButton);
         this.DeviceGroup.Controls.Add(this.textDevice);
         this.DeviceGroup.Controls.Add(this.comboDeviceList);
         this.DeviceGroup.Controls.Add(this.checkSearchROM);
         this.DeviceGroup.Controls.Add(this.textOverdriveDirections);
         this.DeviceGroup.Controls.Add(this.textOverdrive);
         this.DeviceGroup.Controls.Add(this.comboOverdrive);
         this.DeviceGroup.Controls.Add(this.checkOverdrive);
         this.DeviceGroup.Controls.Add(this.textBox6);
         this.DeviceGroup.Controls.Add(this.textBox5);
         this.DeviceGroup.Controls.Add(this.comboSelectMethod);
         this.DeviceGroup.Controls.Add(this.checkResume);
         this.DeviceGroup.Location = new System.Drawing.Point(8, 152);
         this.DeviceGroup.Name = "DeviceGroup";
         this.DeviceGroup.Size = new System.Drawing.Size(472, 328);
         this.DeviceGroup.TabIndex = 1;
         this.DeviceGroup.TabStop = false;
         this.DeviceGroup.Text = "Device Selection Methods";
         this.DeviceGroup.Visible = false;
         // 
         // textOtherDevices
         // 
         this.textOtherDevices.BorderStyle = System.Windows.Forms.BorderStyle.None;
         this.textOtherDevices.Font = new System.Drawing.Font("Microsoft Sans Serif", 8.25F, System.Drawing.FontStyle.Regular, System.Drawing.GraphicsUnit.Point, ((byte)(0)));
         this.textOtherDevices.Location = new System.Drawing.Point(32, 280);
         this.textOtherDevices.Multiline = true;
         this.textOtherDevices.Name = "textOtherDevices";
         this.textOtherDevices.ReadOnly = true;
         this.textOtherDevices.Size = new System.Drawing.Size(320, 16);
         this.textOtherDevices.TabIndex = 46;
         this.textOtherDevices.Text = "(0) non-DS2431 Devices found on network";
         // 
         // refreshButton
         // 
         this.refreshButton.Location = new System.Drawing.Point(280, 240);
         this.refreshButton.Name = "refreshButton";
         this.refreshButton.Size = new System.Drawing.Size(112, 32);
         this.refreshButton.TabIndex = 45;
         this.refreshButton.Text = "Refresh Selection";
         this.refreshButton.Click += new System.EventHandler(this.refreshButton_Click);
         // 
         // textDevice
         // 
         this.textDevice.BorderStyle = System.Windows.Forms.BorderStyle.None;
         this.textDevice.Location = new System.Drawing.Point(32, 232);
         this.textDevice.Multiline = true;
         this.textDevice.Name = "textDevice";
         this.textDevice.ReadOnly = true;
         this.textDevice.Size = new System.Drawing.Size(184, 16);
         this.textDevice.TabIndex = 44;
         this.textDevice.Text = "DS2431 Device Selection";
         // 
         // comboDeviceList
         // 
         this.comboDeviceList.DropDownStyle = System.Windows.Forms.ComboBoxStyle.DropDownList;
         this.comboDeviceList.Location = new System.Drawing.Point(32, 248);
         this.comboDeviceList.Name = "comboDeviceList";
         this.comboDeviceList.Size = new System.Drawing.Size(208, 21);
         this.comboDeviceList.TabIndex = 43;
         this.comboDeviceList.SelectedIndexChanged += new System.EventHandler(this.comboDeviceList_SelectedIndexChanged);
         // 
         // checkSearchROM
         // 
         this.checkSearchROM.Checked = true;
         this.checkSearchROM.CheckState = System.Windows.Forms.CheckState.Checked;
         this.checkSearchROM.Location = new System.Drawing.Point(32, 200);
         this.checkSearchROM.Name = "checkSearchROM";
         this.checkSearchROM.Size = new System.Drawing.Size(408, 32);
         this.checkSearchROM.TabIndex = 42;
         this.checkSearchROM.Text = "Use Search-ROM to find first available DS2431 (Recommended)";
         this.checkSearchROM.CheckedChanged += new System.EventHandler(this.checkBox1_CheckedChanged);
         // 
         // textOverdriveDirections
         // 
         this.textOverdriveDirections.BorderStyle = System.Windows.Forms.BorderStyle.None;
         this.textOverdriveDirections.Location = new System.Drawing.Point(216, 152);
         this.textOverdriveDirections.Multiline = true;
         this.textOverdriveDirections.Name = "textOverdriveDirections";
         this.textOverdriveDirections.ReadOnly = true;
         this.textOverdriveDirections.Size = new System.Drawing.Size(232, 40);
         this.textOverdriveDirections.TabIndex = 41;
         this.textOverdriveDirections.Text = "Overdrive selection will only be done once to get device into overdrive";
         this.textOverdriveDirections.Visible = false;
         // 
         // textOverdrive
         // 
         this.textOverdrive.BorderStyle = System.Windows.Forms.BorderStyle.None;
         this.textOverdrive.Location = new System.Drawing.Point(32, 144);
         this.textOverdrive.Multiline = true;
         this.textOverdrive.Name = "textOverdrive";
         this.textOverdrive.ReadOnly = true;
         this.textOverdrive.Size = new System.Drawing.Size(96, 16);
         this.textOverdrive.TabIndex = 40;
         this.textOverdrive.Text = "Overdrive Selection";
         this.textOverdrive.Visible = false;
         // 
         // comboOverdrive
         // 
         this.comboOverdrive.DropDownStyle = System.Windows.Forms.ComboBoxStyle.DropDownList;
         this.comboOverdrive.Items.AddRange(new object[] {
            "Overdrive Match-ROM",
            "Overdrive Skip-ROM"});
         this.comboOverdrive.Location = new System.Drawing.Point(32, 160);
         this.comboOverdrive.Name = "comboOverdrive";
         this.comboOverdrive.Size = new System.Drawing.Size(152, 21);
         this.comboOverdrive.TabIndex = 39;
         this.comboOverdrive.Visible = false;
         this.comboOverdrive.SelectedIndexChanged += new System.EventHandler(this.comboBox2_SelectedIndexChanged);
         // 
         // checkOverdrive
         // 
         this.checkOverdrive.Location = new System.Drawing.Point(32, 112);
         this.checkOverdrive.Name = "checkOverdrive";
         this.checkOverdrive.Size = new System.Drawing.Size(248, 24);
         this.checkOverdrive.TabIndex = 38;
         this.checkOverdrive.Text = "Use \'Overdrive\' speed when possible";
         this.checkOverdrive.CheckedChanged += new System.EventHandler(this.checkOverdrive_CheckedChanged);
         // 
         // textBox6
         // 
         this.textBox6.BorderStyle = System.Windows.Forms.BorderStyle.None;
         this.textBox6.Location = new System.Drawing.Point(152, 48);
         this.textBox6.Multiline = true;
         this.textBox6.Name = "textBox6";
         this.textBox6.ReadOnly = true;
         this.textBox6.Size = new System.Drawing.Size(312, 24);
         this.textBox6.TabIndex = 37;
         this.textBox6.Text = "WARNING, use SKIP-ROM only on networks with only 1 device!";
         // 
         // textBox5
         // 
         this.textBox5.BorderStyle = System.Windows.Forms.BorderStyle.None;
         this.textBox5.Location = new System.Drawing.Point(32, 24);
         this.textBox5.Multiline = true;
         this.textBox5.Name = "textBox5";
         this.textBox5.ReadOnly = true;
         this.textBox5.Size = new System.Drawing.Size(120, 16);
         this.textBox5.TabIndex = 31;
         this.textBox5.Text = "ROM Selection Method";
         // 
         // comboSelectMethod
         // 
         this.comboSelectMethod.DropDownStyle = System.Windows.Forms.ComboBoxStyle.DropDownList;
         this.comboSelectMethod.Items.AddRange(new object[] {
            "Match-ROM",
            "Skip-ROM"});
         this.comboSelectMethod.Location = new System.Drawing.Point(32, 40);
         this.comboSelectMethod.Name = "comboSelectMethod";
         this.comboSelectMethod.Size = new System.Drawing.Size(104, 21);
         this.comboSelectMethod.TabIndex = 30;
         this.comboSelectMethod.SelectedIndexChanged += new System.EventHandler(this.comboSelectMethod_SelectedIndexChanged);
         // 
         // checkResume
         // 
         this.checkResume.Location = new System.Drawing.Point(32, 80);
         this.checkResume.Name = "checkResume";
         this.checkResume.Size = new System.Drawing.Size(248, 24);
         this.checkResume.TabIndex = 0;
         this.checkResume.Text = "Use \'Resume\' command when possible";
         // 
         // AdapterGroup
         // 
         this.AdapterGroup.Controls.Add(this.autoOpenCheck);
         this.AdapterGroup.Controls.Add(this.textBox4);
         this.AdapterGroup.Controls.Add(this.textAdapterOpen);
         this.AdapterGroup.Controls.Add(this.buttonOpenAdapter);
         this.AdapterGroup.Controls.Add(this.portCombo);
         this.AdapterGroup.Controls.Add(this.textAdapterPartNum);
         this.AdapterGroup.Controls.Add(this.textBox3);
         this.AdapterGroup.Controls.Add(this.textBox2);
         this.AdapterGroup.Controls.Add(this.textBox1);
         this.AdapterGroup.Controls.Add(this.adapterCombo);
         this.AdapterGroup.Location = new System.Drawing.Point(8, 8);
         this.AdapterGroup.Name = "AdapterGroup";
         this.AdapterGroup.Size = new System.Drawing.Size(472, 136);
         this.AdapterGroup.TabIndex = 0;
         this.AdapterGroup.TabStop = false;
         this.AdapterGroup.Text = "1-Wire Adapter";
         // 
         // autoOpenCheck
         // 
         this.autoOpenCheck.Location = new System.Drawing.Point(376, 88);
         this.autoOpenCheck.Name = "autoOpenCheck";
         this.autoOpenCheck.Size = new System.Drawing.Size(88, 24);
         this.autoOpenCheck.TabIndex = 37;
         this.autoOpenCheck.Text = "Auto-Open";
         // 
         // textBox4
         // 
         this.textBox4.BorderStyle = System.Windows.Forms.BorderStyle.None;
         this.textBox4.Location = new System.Drawing.Point(184, 80);
         this.textBox4.Multiline = true;
         this.textBox4.Name = "textBox4";
         this.textBox4.ReadOnly = true;
         this.textBox4.Size = new System.Drawing.Size(96, 16);
         this.textBox4.TabIndex = 36;
         this.textBox4.Text = "Status";
         // 
         // textAdapterOpen
         // 
         this.textAdapterOpen.BackColor = System.Drawing.SystemColors.HighlightText;
         this.textAdapterOpen.BorderStyle = System.Windows.Forms.BorderStyle.None;
         this.textAdapterOpen.ForeColor = System.Drawing.Color.Yellow;
         this.textAdapterOpen.Location = new System.Drawing.Point(184, 96);
         this.textAdapterOpen.Multiline = true;
         this.textAdapterOpen.Name = "textAdapterOpen";
         this.textAdapterOpen.ReadOnly = true;
         this.textAdapterOpen.Size = new System.Drawing.Size(168, 16);
         this.textAdapterOpen.TabIndex = 35;
         // 
         // buttonOpenAdapter
         // 
         this.buttonOpenAdapter.Location = new System.Drawing.Point(32, 80);
         this.buttonOpenAdapter.Name = "buttonOpenAdapter";
         this.buttonOpenAdapter.Size = new System.Drawing.Size(120, 32);
         this.buttonOpenAdapter.TabIndex = 34;
         this.buttonOpenAdapter.Text = "Open Adapter/Port";
         this.buttonOpenAdapter.Click += new System.EventHandler(this.button1_Click);
         // 
         // portCombo
         // 
         this.portCombo.DropDownStyle = System.Windows.Forms.ComboBoxStyle.DropDownList;
         this.portCombo.Location = new System.Drawing.Point(280, 40);
         this.portCombo.Name = "portCombo";
         this.portCombo.Size = new System.Drawing.Size(96, 21);
         this.portCombo.TabIndex = 33;
         // 
         // textAdapterPartNum
         // 
         this.textAdapterPartNum.BackColor = System.Drawing.SystemColors.HighlightText;
         this.textAdapterPartNum.BorderStyle = System.Windows.Forms.BorderStyle.None;
         this.textAdapterPartNum.Location = new System.Drawing.Point(152, 40);
         this.textAdapterPartNum.Multiline = true;
         this.textAdapterPartNum.Name = "textAdapterPartNum";
         this.textAdapterPartNum.ReadOnly = true;
         this.textAdapterPartNum.Size = new System.Drawing.Size(96, 16);
         this.textAdapterPartNum.TabIndex = 32;
         this.textAdapterPartNum.TextChanged += new System.EventHandler(this.textBox4_TextChanged);
         // 
         // textBox3
         // 
         this.textBox3.BorderStyle = System.Windows.Forms.BorderStyle.None;
         this.textBox3.Location = new System.Drawing.Point(280, 24);
         this.textBox3.Multiline = true;
         this.textBox3.Name = "textBox3";
         this.textBox3.ReadOnly = true;
         this.textBox3.Size = new System.Drawing.Size(96, 16);
         this.textBox3.TabIndex = 31;
         this.textBox3.Text = "Adapter Port";
         // 
         // textBox2
         // 
         this.textBox2.BorderStyle = System.Windows.Forms.BorderStyle.None;
         this.textBox2.Location = new System.Drawing.Point(152, 24);
         this.textBox2.Multiline = true;
         this.textBox2.Name = "textBox2";
         this.textBox2.ReadOnly = true;
         this.textBox2.Size = new System.Drawing.Size(96, 16);
         this.textBox2.TabIndex = 30;
         this.textBox2.Text = "Adapter Part #";
         // 
         // textBox1
         // 
         this.textBox1.BorderStyle = System.Windows.Forms.BorderStyle.None;
         this.textBox1.Location = new System.Drawing.Point(32, 24);
         this.textBox1.Multiline = true;
         this.textBox1.Name = "textBox1";
         this.textBox1.ReadOnly = true;
         this.textBox1.Size = new System.Drawing.Size(96, 16);
         this.textBox1.TabIndex = 29;
         this.textBox1.Text = "Adapter Port Type";
         // 
         // adapterCombo
         // 
         this.adapterCombo.DropDownStyle = System.Windows.Forms.ComboBoxStyle.DropDownList;
         this.adapterCombo.Items.AddRange(new object[] {
            "USB",
            "Serial (COM)",
            "Parallel (LPT)"});
         this.adapterCombo.Location = new System.Drawing.Point(32, 40);
         this.adapterCombo.Name = "adapterCombo";
         this.adapterCombo.Size = new System.Drawing.Size(104, 21);
         this.adapterCombo.TabIndex = 0;
         this.adapterCombo.SelectedIndexChanged += new System.EventHandler(this.adapterCombo_SelectedIndexChanged);
         // 
         // tabRaw
         // 
         this.tabRaw.Controls.Add(this.lowLevelBox);
         this.tabRaw.Controls.Add(this.oneWireIOTextBox);
         this.tabRaw.Controls.Add(this.ROMGroup);
         this.tabRaw.Location = new System.Drawing.Point(4, 22);
         this.tabRaw.Name = "tabRaw";
         this.tabRaw.Size = new System.Drawing.Size(488, 542);
         this.tabRaw.TabIndex = 1;
         this.tabRaw.Text = "1-Wire Communication";
         // 
         // lowLevelBox
         // 
         this.lowLevelBox.Controls.Add(this.button12VPulse);
         this.lowLevelBox.Controls.Add(this.checkBoxHideLog);
         this.lowLevelBox.Controls.Add(this.repeatScriptCount);
         this.lowLevelBox.Controls.Add(this.buttonRepeatScript);
         this.lowLevelBox.Controls.Add(this.button4);
         this.lowLevelBox.Controls.Add(this.button3);
         this.lowLevelBox.Controls.Add(this.readBitButton);
         this.lowLevelBox.Controls.Add(this.comboSpeed);
         this.lowLevelBox.Controls.Add(this.resetButton);
         this.lowLevelBox.Controls.Add(this.label1);
         this.lowLevelBox.Controls.Add(this.readBytesButton);
         this.lowLevelBox.Controls.Add(this.readBytesCount);
         this.lowLevelBox.Controls.Add(this.bytesTextBox);
         this.lowLevelBox.Controls.Add(this.touchByteButton);
         this.lowLevelBox.Controls.Add(this.oneBitButton);
         this.lowLevelBox.Controls.Add(this.zeroBitButton);
         this.lowLevelBox.Controls.Add(this.strongPullupButton);
         this.lowLevelBox.Controls.Add(this.strongPullupBitButton);
         this.lowLevelBox.Controls.Add(this.setPowerNormalButton);
         this.lowLevelBox.Controls.Add(this.label13);
         this.lowLevelBox.Location = new System.Drawing.Point(8, 8);
         this.lowLevelBox.Name = "lowLevelBox";
         this.lowLevelBox.Size = new System.Drawing.Size(472, 227);
         this.lowLevelBox.TabIndex = 26;
         this.lowLevelBox.TabStop = false;
         this.lowLevelBox.Text = "+";
         // 
         // checkBoxHideLog
         // 
         this.checkBoxHideLog.AutoSize = true;
         this.checkBoxHideLog.Location = new System.Drawing.Point(397, 40);
         this.checkBoxHideLog.Name = "checkBoxHideLog";
         this.checkBoxHideLog.Size = new System.Drawing.Size(69, 17);
         this.checkBoxHideLog.TabIndex = 26;
         this.checkBoxHideLog.Text = "Hide Log";
         this.checkBoxHideLog.UseVisualStyleBackColor = true;
         this.checkBoxHideLog.CheckedChanged += new System.EventHandler(this.checkBoxHideLog_CheckedChanged);
         // 
         // repeatScriptCount
         // 
         this.repeatScriptCount.Font = new System.Drawing.Font("Courier New", 9F);
         this.repeatScriptCount.Location = new System.Drawing.Point(419, 13);
         this.repeatScriptCount.Maximum = new decimal(new int[] {
            256,
            0,
            0,
            0});
         this.repeatScriptCount.Name = "repeatScriptCount";
         this.repeatScriptCount.Size = new System.Drawing.Size(48, 21);
         this.repeatScriptCount.TabIndex = 25;
         this.repeatScriptCount.TextAlign = System.Windows.Forms.HorizontalAlignment.Center;
         this.repeatScriptCount.Value = new decimal(new int[] {
            1,
            0,
            0,
            0});
         // 
         // buttonRepeatScript
         // 
         this.buttonRepeatScript.Enabled = false;
         this.buttonRepeatScript.Location = new System.Drawing.Point(296, 13);
         this.buttonRepeatScript.Name = "buttonRepeatScript";
         this.buttonRepeatScript.Size = new System.Drawing.Size(117, 24);
         this.buttonRepeatScript.TabIndex = 24;
         this.buttonRepeatScript.Text = "Repeat Last Script";
         this.buttonRepeatScript.Click += new System.EventHandler(this.buttonRepeatScript_Click);
         // 
         // button4
         // 
         this.button4.Location = new System.Drawing.Point(199, 190);
         this.button4.Name = "button4";
         this.button4.Size = new System.Drawing.Size(136, 24);
         this.button4.TabIndex = 23;
         this.button4.Text = "POR (1 second)";
         this.button4.Click += new System.EventHandler(this.button4_Click);
         // 
         // button3
         // 
         this.button3.Location = new System.Drawing.Point(55, 190);
         this.button3.Name = "button3";
         this.button3.Size = new System.Drawing.Size(136, 24);
         this.button3.TabIndex = 22;
         this.button3.Text = "7V Pulse (100ms)";
         this.button3.Click += new System.EventHandler(this.button3_Click);
         // 
         // label13
         // 
         this.label13.Location = new System.Drawing.Point(6, 147);
         this.label13.Name = "label13";
         this.label13.Size = new System.Drawing.Size(55, 32);
         this.label13.TabIndex = 21;
         this.label13.Text = "Power Delivery:";
         this.label13.TextAlign = System.Drawing.ContentAlignment.MiddleLeft;
         // 
         // readBitButton
         // 
         this.readBitButton.Location = new System.Drawing.Point(254, 48);
         this.readBitButton.Name = "readBitButton";
         this.readBitButton.Size = new System.Drawing.Size(88, 24);
         this.readBitButton.TabIndex = 19;
         this.readBitButton.Text = "Read Bit";
         this.readBitButton.Click += new System.EventHandler(this.readBitButton_Click);
         // 
         // comboSpeed
         // 
         this.comboSpeed.Items.AddRange(new object[] {
            "Speed - Standard",
            "Speed - Overdrive"});
         this.comboSpeed.Location = new System.Drawing.Point(112, 16);
         this.comboSpeed.Name = "comboSpeed";
         this.comboSpeed.Size = new System.Drawing.Size(168, 21);
         this.comboSpeed.TabIndex = 3;
         this.comboSpeed.SelectedIndexChanged += new System.EventHandler(this.comboSpeed_SelectedIndexChanged);
         // 
         // button12VPulse
         // 
         this.button12VPulse.Location = new System.Drawing.Point(343, 190);
         this.button12VPulse.Name = "button12VPulse";
         this.button12VPulse.Size = new System.Drawing.Size(104, 24);
         this.button12VPulse.TabIndex = 27;
         this.button12VPulse.Text = "12V 1-Wire Pulse";
         this.button12VPulse.Click += new System.EventHandler(this.button12VPulse_Click);
         // 
         // OneWireIO
         // 
         this.AutoScaleBaseSize = new System.Drawing.Size(5, 13);
         this.ClientSize = new System.Drawing.Size(512, 593);
         this.Controls.Add(this.tabControl);
         this.Controls.Add(this.StatusBar);
         this.Icon = ((System.Drawing.Icon)(resources.GetObject("$this.Icon")));
         this.Menu = this.mainMenu1;
         this.MinimumSize = new System.Drawing.Size(424, 450);
         this.Name = "OneWireIO";
         this.Text = "DS2431 Evaluation Program";
         this.Load += new System.EventHandler(this.OneWireIO_Load);
         this.Resize += new System.EventHandler(this.OneWireIO_Resize);
         ((System.ComponentModel.ISupportInitialize)(this.AdapterStatus)).EndInit();
         ((System.ComponentModel.ISupportInitialize)(this.Status)).EndInit();
         ((System.ComponentModel.ISupportInitialize)(this.readBytesCount)).EndInit();
         this.ROMGroup.ResumeLayout(false);
         this.ROMGroup.PerformLayout();
         this.tabControl.ResumeLayout(false);
         this.setupTab.ResumeLayout(false);
         this.KeyGroup.ResumeLayout(false);
         this.DeviceGroup.ResumeLayout(false);
         this.DeviceGroup.PerformLayout();
         this.AdapterGroup.ResumeLayout(false);
         this.AdapterGroup.PerformLayout();
         this.tabRaw.ResumeLayout(false);
         this.lowLevelBox.ResumeLayout(false);
         this.lowLevelBox.PerformLayout();
         ((System.ComponentModel.ISupportInitialize)(this.repeatScriptCount)).EndInit();
         this.ResumeLayout(false);

      }
      #endregion

      /// <summary>
      /// The main entry point for the application.
      /// </summary>
      [STAThread]
      static void Main() 
      {
         Application.Run(new OneWireIO());
      }

      private void resetODMatchROMButton_Click(object sender, System.EventArgs e)
      {
         byte[] rom;
         try
         {
            statusWrite("",TX_COLOR);
            statusWrite("// Overdrive Match ROM, FORMAT: 69h (command), change to overdrive, Registration Number (8 bytes)",COMMENT_COLOR);
            comboSpeed.SelectedIndex = 0;
            if (OWReset())
            {
               adapter.PutByte(0x69); // Match ROM cmd
               statusWrite("69 ",TX_COLOR);
               comboSpeed.SelectedIndex = 1;
               rom = FromHex(textBoxROM.Text);   
   
               if (rom.Length != 8)
               {
                  MessageBox.Show("WARNING, There must be 8 hex bytes in the ROM text box to use Match-ROM." + 
                     System.Environment.NewLine + "For example: 2F B1 36 07 00 00 00 03.", AppTitle,
                     MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                  return;
               }

               adapter.DataBlock(rom, 0, 8);
               statusWrite(ToHex(rom, 0, 8), TX_COLOR);
               statusWrite("",RX_COLOR);
            }
         }
         catch(Exception ex)
         {
            statusWrite("<<" + ex.Message + ">>",ERROR_COLOR); 
            statusWrite("",ERROR_COLOR);
         } 
      }

      private void resetMatchROMButton_Click(object sender, System.EventArgs e)
      {
         byte[] rom;
         try
         {
            statusWrite("",TX_COLOR);
            statusWrite("// Match ROM, FORMAT: 55h (command), Registration Number (8 bytes)",COMMENT_COLOR);
            if (OWReset())
            {
               adapter.PutByte(0x55); // Match ROM cmd
               statusWrite("55 ",TX_COLOR);
               rom = FromHex(textBoxROM.Text);
               if (rom.Length != 8)
               {
                  MessageBox.Show("WARNING, There must be 8 hex bytes in the ROM text box to use Match-ROM." + 
                     System.Environment.NewLine + "For example: 2F B1 36 07 00 00 00 03.", AppTitle,
                     MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                  return;
               }

               adapter.DataBlock(rom, 0, 8);
               statusWrite(ToHex(rom, 0, 8), TX_COLOR);
               statusWrite("",RX_COLOR);
            }
         }
         catch(Exception ex)
         {
            statusWrite("<<" + ex.Message + ">>",ERROR_COLOR); 
            statusWrite("",ERROR_COLOR);
         } 
      }

      private void resetSkipROMbutton_Click(object sender, System.EventArgs e)
      {
         try
         {
            statusWrite("",TX_COLOR);
            statusWrite("// Skip-ROM, FORMAT: CCh (command)",COMMENT_COLOR);
            if (OWReset())
            {
               adapter.PutByte(0xCC); // Skip ROM cmd
               statusWrite("CC ",TX_COLOR);
            }
         }
         catch(Exception ex)
         {
            statusWrite("<<" + ex.Message + ">>",ERROR_COLOR); 
            statusWrite("",ERROR_COLOR);
         }  
      }

      public bool OWReset()
      {
         OWResetResult rslt;

         try
         {
            statusWrite("",TX_COLOR);
            // show current speed
            if (comboSpeed.SelectedIndex == 1)
            {
               statusWrite("<OVR> ",TX_COLOR);
               //adapter.Speed = OWSpeed.SPEED_OVERDRIVE;
            }
            else
            {
               statusWrite("<STD> ",TX_COLOR);
               //adapter.Speed = OWSpeed.SPEED_REGULAR;
            }
            statusWrite("R",TX_COLOR);
            rslt = adapter.Reset();
            if ((rslt == OWResetResult.RESET_PRESENCE) || (rslt == OWResetResult.RESET_ALARM))
            {
               statusWrite("P ",RX_COLOR);
               return true;
            }
            else
            {
               statusWrite("N ",RX_COLOR);
               return false;
            }
         }
         catch(Exception ex)
         {
            statusWrite("<<" + ex.Message + ">>",ERROR_COLOR); 
            statusWrite("",ERROR_COLOR);
            return false;
         }
      }

      private void resetODSkipROMButton_Click(object sender, System.EventArgs e)
      {
         try
         {
            statusWrite("",TX_COLOR);
            statusWrite("// Overdrive Skip-ROM, FORMAT: 3Ch (command), change to overdrive",COMMENT_COLOR);
            comboSpeed.SelectedIndex = 0;
            if (OWReset())
            {
               adapter.PutByte(0x3C); // Skip ROM cmd
               statusWrite("3C ",TX_COLOR);
               comboSpeed.SelectedIndex = 1;
            }
         }
         catch(Exception ex)
         {
            statusWrite("<<" + ex.Message + ">>",ERROR_COLOR); 
            statusWrite("",ERROR_COLOR);
         } 
      }

      private void onewireSearchButton_Click(object sender, System.EventArgs e)
      {
         byte[] cmd = FromHex(searchCommandText.Text);
         if (cmd.Length < 1)
         {
            MessageBox.Show("WARNING, There must be a Search Command in the search command text box." + 
               System.Environment.NewLine + "Typical choices are: F0 or EC.", AppTitle,
               MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
         }
         else
         {
            statusWrite("",COMMENT_COLOR);
            if (OWSearch(true,comboSpeed.SelectedIndex == 1,cmd[0]))
               textBoxROM.Text = ToHex(ROM_NO,0 ,8);
         }
      }

      private void touchByteButton_Click(object sender, System.EventArgs e)
      {
         bytesTextBox.Text = bytesTextBox.Text.Trim();
         if(!bytesTextBox.Items.Contains(bytesTextBox.Text))
            bytesTextBox.Items.Insert(0, bytesTextBox.Text);

         try
         {
            if(bytesTextBox.Text.Length>0)
            {
               byte[] block = FromHex(bytesTextBox.Text);
            
               for (int i = 0; i < block.Length; i++)
               {
                  statusWrite(block[i].ToString("X2") + " ",TX_COLOR);
                  adapter.PutByte(block[i]);   

                  if (afterbyte)
                  {
                     statusWrite("<SP_ON> ",TX_COLOR);
                     afterbyte = false;
                  }
               }
            }
         }
         catch(Exception ex)
         {
            statusWrite("<<" + ex.Message + ">>",ERROR_COLOR); 
            statusWrite("",ERROR_COLOR);
         }
      }

      private void resetTouchBytesButton_Click(object sender, System.EventArgs e)
      {
         bytesTextBox.Text = bytesTextBox.Text.Trim();
         if(!bytesTextBox.Items.Contains(bytesTextBox.Text))
            bytesTextBox.Items.Insert(0, bytesTextBox.Text);
         if(adapter!=null)
         {
            try
            {
               adapter.Speed = OWSpeed.SPEED_REGULAR;

               addText("Reset: " + adapter.Speed + " - " + adapter.Reset(), Color.Blue);
               
               if(bytesTextBox.Text.Length>0)
               {
                  byte[] block = FromHex(bytesTextBox.Text);
                  
                  addText("Send: " + adapter.Speed, Color.Green);
                  addText(ToHex(block, 0, block.Length), Color.Green);
                  
                  adapter.DataBlock(block, 0, block.Length);
                  
                  addText("Recv: " + adapter.Speed, Color.Black);
                  addText(ToHex(block, 0, block.Length), Color.Black);
               }
            }
            catch(Exception ex)
            {
               addText("Failure: " + ex.Message + "\n" + ex.StackTrace + "\n" + ((ex.InnerException!=null)?ex.InnerException.Message:""), Color.Red);
            }
         }
         else
            addText("No Adapter", Color.Red);
      }

      private void odResetTouchBytes_Click(object sender, System.EventArgs e)
      {
         
      }

      private void readBytesButton_Click(object sender, System.EventArgs e)
      {
         int data;
         try
         {
            for (int i = 0; i < readBytesCount.Value; i++)
            {
               data = adapter.GetByte();
               statusWrite("[" + data.ToString("X2") + "] ",RX_COLOR);
               if (afterbyte)
               {
                  statusWrite("<SP_ON> ",TX_COLOR);
                  afterbyte = false;
               }
            }

         }
         catch(Exception ex)
         {
            statusWrite("<<" + ex.Message + ">>",ERROR_COLOR); 
            statusWrite("",ERROR_COLOR);
         }
      }

      private void addText(string text, Color col)
      {
         try
         {
            oneWireIOTextBox.SelectionColor = col;
            //oneWireIOTextBox.AppendText(text + "\r\n");
            oneWireIOTextBox.AppendText(text + System.Environment.NewLine);
            //oneWireIOTextBox.SelectionStart = oneWireIOTextBox.Text.Length - 1;
            oneWireIOTextBox.Focus();
            oneWireIOTextBox.ScrollToCaret();
            bytesTextBox.Focus();
            bytesTextBox.SelectAll();
         }
         catch(Exception e)
         {
            Console.Out.WriteLine(e);
         }
      }

      private void statusWrite(string str, Color col)
      {
         bool returnfocus_combo=false;

         if (hide_log_mode)
            return;

         try
         {
            if (str.Length == 0)
            {
                  oneWireIOTextBox.AppendText(System.Environment.NewLine);
            }
            else
            {
               oneWireIOTextBox.SelectionColor = col;

               if (col == ERROR_COLOR)
                  oneWireIOTextBox.SelectionBackColor = Color.Yellow; 
               else
                  oneWireIOTextBox.SelectionBackColor = Color.White; 

               if (newline)
               {
                     oneWireIOTextBox.AppendText(str + System.Environment.NewLine);
               }
               else
               {
                     oneWireIOTextBox.AppendText(str);
               }

               if (!mask_scroll)
               {
                  oneWireIOTextBox.Focus();
                  oneWireIOTextBox.ScrollToCaret();
               }

               newline = false;
            }
         }
         catch(Exception e)
         {
            Console.Out.WriteLine(e);
         }
      }

      private void statusWriteLN(string str, Color col)
      {
         bool returnfocus_combo=false;

         if (hide_log_mode)
            return;

         try
         {  
               oneWireIOTextBox.SelectionColor = col;

               if (col == ERROR_COLOR)
                  oneWireIOTextBox.SelectionBackColor = Color.Yellow;
               else
                  oneWireIOTextBox.SelectionBackColor = Color.White;

               oneWireIOTextBox.AppendText(str + System.Environment.NewLine);

               if (!mask_scroll)
               {
                  oneWireIOTextBox.Focus();
                  oneWireIOTextBox.ScrollToCaret();
               }

            newline = false;
         }
         catch(Exception e)
         {
            Console.Out.WriteLine(e);
         }
      }

      private void clearTextButton_Click(object sender, System.EventArgs e)
      {
      }

      private void pureDS9097U_Click(object sender, System.EventArgs e)
      {

      }
      private void DS9097U_Click(object sender, System.EventArgs e)
      {

      }
      private void DS1410E_Click(object sender, System.EventArgs e)
      {

      }
      private void DS9490_Click(object sender, System.EventArgs e)
      {

      }
      private void DS9097E_Click(object sender, System.EventArgs e)
      {

      }

      private void setAdapter(string adapterName, string portName)
      {
         try
         {
            if(adapter!=null)
            {
               adapter.EndExclusive();
               adapter.Dispose();
               adapter = null;
            }

            adapter = AccessProvider.GetAdapter(adapterName, portName);

            if(adapter!=null)
            {
               AdapterStatus.Text = adapter.ToString();
               adapter.BeginExclusive(true);

               adapter.Speed = OWSpeed.SPEED_REGULAR;
               adapter.Reset();
               adapter.PutByte(0x3C);
               adapter.Speed = OWSpeed.SPEED_OVERDRIVE;
               adapter.PutByte(0x3C);

               adapter.Speed = OWSpeed.SPEED_REGULAR;
               adapter.Reset();
               adapter.PutByte(0x3C);
               adapter.PutByte(0x3C);

               adapter.Speed = OWSpeed.SPEED_REGULAR;
               adapter.Reset();
               adapter.PutByte(0x3C);
               adapter.Speed = OWSpeed.SPEED_OVERDRIVE;
            }

            //deviceAddressList.Items.Clear();
            //deviceAddressList.Text = "";
         }
         catch(Exception ex)
         {
            Console.WriteLine("Failed to set adapter: " + ex.Message);
            Console.WriteLine("StackTrace = " + ex.StackTrace);
            addText("Failure: " + ex.Message + "\n" + ex.StackTrace + "\n" + ((ex.InnerException!=null)?ex.InnerException.Message:""), Color.Red);
            AdapterStatus.Text = "Not Loaded";
            if(adapter!=null)
               adapter.Dispose();
            adapter = null;
         }		
      }

      private void menuItem2_Click(object sender, System.EventArgs e)
      {
         this.Close();
      }

      private void resetButton_Click(object sender, System.EventArgs e)
      {
         OWReset();
      }

      private void odResetButton_Click(object sender, System.EventArgs e)
      {
         if(adapter!=null)
         {
            try
            {
               adapter.Speed = OWSpeed.SPEED_OVERDRIVE;

               addText("Reset: " + adapter.Speed + " - " + adapter.Reset(), Color.Blue);
            }
            catch(Exception ex)
            {
               addText("Failure: " + ex.Message + "\n" + ex.StackTrace + "\n" + ((ex.InnerException!=null)?ex.InnerException.Message:""), Color.Red);
            }
         }
         else
            addText("No Adapter", Color.Red);
      }

      private void zeroBitButton_Click(object sender, System.EventArgs e)
      {
         try
         {
            adapter.PutBit(false);
            statusWrite("0 ",TX_COLOR);

            if (afterbit)
            {
               statusWrite("<SP_ON> ",TX_COLOR);
               afterbit = false;
            }
         }
         catch(Exception ex)
         {
            statusWrite("<<" + ex.Message + ">>",ERROR_COLOR); 
            statusWrite("",ERROR_COLOR);         
         }      
      }

      private void oneBitButton_Click(object sender, System.EventArgs e)
      {
         try
         {
            adapter.PutBit(true);
            statusWrite("1 ",TX_COLOR);

            if (afterbit)
            {
               statusWrite("<SP_ON> ",TX_COLOR);
               afterbit = false;
            }
         }
         catch(Exception ex)
         {
            statusWrite("<<" + ex.Message + ">>",ERROR_COLOR); 
            statusWrite("",ERROR_COLOR);         
         }
      }

      private void menuItem25_Click(object sender, System.EventArgs e)
      {
         
      }

      private void menuItem26_Click(object sender, System.EventArgs e)
      {
         
      }

      private void menuItem27_Click(object sender, System.EventArgs e)
      {
         
      }

      private void startAfterByteRadioButton_CheckedChanged(object sender, System.EventArgs e)
      {
      }

      private void startAfterBitRadioButton_CheckedChanged(object sender, System.EventArgs e)
      {
      }

      private void strongPullupButton_Click(object sender, System.EventArgs e)
      {
         statusWrite("",COMMENT_COLOR);
         statusWriteLN("// Set power delivery after next byte", COMMENT_COLOR);         
         
         try
         {
            adapter.StartPowerDelivery(OWPowerStart.CONDITION_AFTER_BYTE);
            afterbyte = true;
         }
         catch(Exception ex)
         {
            statusWrite("<<" + ex.Message + ">>",ERROR_COLOR); 
            statusWrite("",ERROR_COLOR);
         }
      }

      private void setPowerNormalButton_Click(object sender, System.EventArgs e)
      {
         try
         {
            statusWrite("<SP_OFF> ",TX_COLOR);
            adapter.SetPowerNormal();
         }
         catch(Exception ex)
         {
            statusWrite("<<" + ex.Message + ">>",ERROR_COLOR); 
            statusWrite("",ERROR_COLOR);
         }
      }


      private static string ToHex(byte[] buff, int off, int len)
      {
         System.Text.StringBuilder sb = new System.Text.StringBuilder(buff.Length*3);
         sb.Append(buff[off].ToString("X2"));
         for(int i=1; i<len; i++)
         {
            sb.Append(" ");
            sb.Append(buff[off+i].ToString("X2"));
         }
         return sb.ToString();
      }

      private static byte[] FromHex(string s)
      {
         s = System.Text.RegularExpressions.Regex.Replace(s.ToUpper(), "[^0-9A-F]", "");
         byte[] b = new byte[s.Length / 2];
         for (int i = 0; i < s.Length; i += 2)
            b[i / 2] = byte.Parse(s.Substring(i, 2), 
               System.Globalization.NumberStyles.AllowHexSpecifier);

         return b;
      }

      private void label1_Click(object sender, System.EventArgs e)
      {
         String st = "";
         st = oneWireIOTextBox.Lines[0] + System.Environment.NewLine + oneWireIOTextBox.Lines[1];
         Clipboard.SetDataObject(st,true);
      }

      private void comboMemorySelector_SelectedIndexChanged(object sender, System.EventArgs e)
      {
      }

      private void menuItem33_Click(object sender, System.EventArgs e)
      {
         oneWireIOTextBox.SelectAll();
         oneWireIOTextBox.Copy();
      }

      private void readButton_Click(object sender, System.EventArgs e)
      {
      }

      private void textBox4_TextChanged(object sender, System.EventArgs e)
      {
      
      }

      private void comboBox2_SelectedIndexChanged(object sender, System.EventArgs e)
      {
         // force check search ROM if match rom is selected
         if (comboOverdrive.SelectedIndex == 0)
         {
            if (!checkSearchROM.Checked)
               checkSearchROM.Checked = true;
         }
      }

      private void tabControl1_SelectedIndexChanged(object sender, System.EventArgs e)
      {
         // check for adapter/port not selected
         if ((tabControl.SelectedIndex != 0) && (adapter == null))
         {
            MessageBox.Show("The 1-Wire Adapter/Port must be opened before proceeding to the Memory or Raw tab." + 
               System.Environment.NewLine + "Click OK to return to the Setup tab.", AppTitle,
               MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
            tabControl.SelectedIndex = 0;
            return;
         }


            comboSpeed.SelectedIndex = 0;
            if (inOverdrive)
            {
               comboSpeed.SelectedIndex = 1;
               comboSpeed.SelectedIndex = 0;
            }
            use_resume = false;
            inOverdrive = false;
            afterbit = false;
            afterbyte = false;
         lasttab = tabControl.SelectedIndex;
      }

      private void adapterCombo_SelectedIndexChanged(object sender, System.EventArgs e)
      {
         if (adapterCombo.SelectedIndex == 0)
         {
            textAdapterPartNum.Text = "DS9490R/B";
            portCombo.Items.Clear(); 
            for (int i = 0; i < 16; i++)
               portCombo.Items.Add("USB" + i.ToString());
            portCombo.SelectedIndex = 1;
            selectedAdapterName = "{DS9490}";
         }
         else if (adapterCombo.SelectedIndex == 1)
         {
            textAdapterPartNum.Text = "DS9097U";

            // populate the port drop down
            string[] commports;
            commports = SerialPort.GetPortNames();
            portCombo.Items.Clear();
            for (int i = 0; i < commports.Length; i++)
               portCombo.Items.Add(commports[i]);

            /*????
            portCombo.Items.Clear(); 
            for (int i = 1; i < 16; i++)
               portCombo.Items.Add("COM" + i.ToString());
            portCombo.SelectedIndex = 0;
             ?????*/
            selectedAdapterName = "{DS9097U}";
         }
         else if (adapterCombo.SelectedIndex == 2)
         {
            textAdapterPartNum.Text = "DS1410E";
            portCombo.Items.Clear(); 
            for (int i = 0; i < 15; i++)
               portCombo.Items.Add("LPT" + i.ToString());
            portCombo.SelectedIndex = 0;
            selectedAdapterName = "{DS1410E}";
         }
      }

      private void button1_Click(object sender, System.EventArgs e)
      {
         OpenAdapterPort();
      }

      public bool OpenAdapterPort()
      {
         textAdapterOpen.Text = "";
         Object selectedItem = portCombo.SelectedItem;

         try
         {
            if (adapter != null)
            {
               adapter.EndExclusive();
               adapter.Dispose();
               adapter = null;
            }

            adapter = AccessProvider.GetAdapter(selectedAdapterName, selectedItem.ToString());
            textAdapterOpen.ForeColor = Color.Green;
            textAdapterOpen.Text = "Success";
            AdapterStatus.Text = adapter.ToString();
            Status.Text = "Normal";

            statusWriteLN("// Adapter: " + textAdapterPartNum.Text + ", Port: " + selectedItem.ToString(), COMMENT_COLOR);

            //??if (checkSearchROM.Checked)
            //???   refreshDeviceList(family_DS28E05, 0, 0);

            return true;
         }
         catch (Exception ex)
         {
            textAdapterOpen.ForeColor = Color.Red;
            textAdapterOpen.Text = "Error: " + ex.Message;
            AdapterStatus.Text = "Adapter Open Failure";
            Status.Text = "ERROR: " + ex.Message;
            adapter = null;
            return false;
         }   

         /*?????????
         textAdapterOpen.Text = "";
         Object selectedItem = portCombo.SelectedItem;

         try
         {
            if(adapter!=null)
            {
               adapter.EndExclusive();
               adapter.Dispose();
               adapter = null;
            }

            adapter = AccessProvider.GetAdapter(selectedAdapterName, selectedItem.ToString());
            textAdapterOpen.ForeColor = Color.Green;
            textAdapterOpen.Text = "Success";
            AdapterStatus.Text = adapter.ToString();
            Status.Text = "Normal";

            statusWriteLN("// Adapter: " + selectedAdapterName + ", Port: " + selectedItem.ToString(),COMMENT_COLOR);

            //??if (checkSearchROM.Checked)
            //???   refreshDeviceList(family_code);

            return true;
         }
         catch(Exception ex)
         {
            textAdapterOpen.ForeColor = Color.Red;
            textAdapterOpen.Text = "Error: " + ex.Message;
            AdapterStatus.Text = "Adapter Open Failure";
            Status.Text = "ERROR: " + ex.Message;
            adapter = null;
            return false;
         }   
          ??????????*/
      }

      private int refreshDeviceList(int family)
      {
         bool rslt;

         try
         {
            selectedDevice = null;
            comboDeviceList.Items.Clear();
            adapter.Speed = OWSpeed.SPEED_REGULAR;

            /* \\//\\//\\// Method using high-level calls
            adapter.TargetAllFamilies();
            adapter.TargetFamily(0x2F);
            byte[] address = new byte[8];
            if(adapter.GetFirstDevice(address, 0))
            {
               do
               {
                  comboDeviceList.Items.Add(ToHex(address, 0, 8));
               }
               while(adapter.GetNextDevice(address, 0));
            }
            if(comboDeviceList.Items.Count>0)
               comboDeviceList.SelectedIndex = 0;
            */

            rslt = OWSearch(true,false,0xF0); 

            other_cnt = 0;
            while (rslt)
            {
               if (family == (ROM_NO[0] & 0x7F))
                  comboDeviceList.Items.Add(ToHex(ROM_NO, 0, 8));
               else
                  other_cnt++;
               rslt = OWSearch(false,false,0xF0);             
            }
      
            textOtherDevices.Text = "(" + other_cnt.ToString() + ") non-DS2431 Devices found on network";
            if(comboDeviceList.Items.Count>0)
               comboDeviceList.SelectedIndex = 0;
            
            return comboDeviceList.Items.Count;
         }
         catch(Exception ex)
         {
            statusWrite("<<" + ex.Message + ">>",ERROR_COLOR); 
            use_resume = false;
            inOverdrive = false;
            statusWrite("",ERROR_COLOR);
            //addText("Failure: " + ex.Message + "\n" + ex.StackTrace + "\n" + ((ex.InnerException!=null)?ex.InnerException.Message:""), Color.Red);
            return 0;
         }

      }


      private void comboDeviceList_SelectedIndexChanged(object sender, System.EventArgs e)
      {
         Object selectedItem = comboDeviceList.SelectedItem;

         selectedDevice = FromHex(selectedItem.ToString());
         textBoxROM.Text = selectedItem.ToString();

         statusWriteLN("// Registration Number selected: " + selectedItem.ToString(), COMMENT_COLOR); 

         use_resume = false;
         inOverdrive = false;
      }

      private void checkOverdrive_CheckedChanged(object sender, System.EventArgs e)
      {
         textOverdrive.Visible = checkOverdrive.Checked;
         textOverdriveDirections.Visible = checkOverdrive.Checked;
         comboOverdrive.Visible = checkOverdrive.Checked;
         // check if need to show search
         if (checkOverdrive.Checked && (comboOverdrive.SelectedIndex == 0))
         {
            if (!checkSearchROM.Checked)
               checkSearchROM.Checked = true;
         }
      }

      private void checkBox1_CheckedChanged(object sender, System.EventArgs e)
      {
         comboDeviceList.Visible = checkSearchROM.Checked;
         textDevice.Visible = checkSearchROM.Checked;
         refreshButton.Visible = checkSearchROM.Checked;
         if (checkSearchROM.Checked)
            textOtherDevices.Text = "";
         textOtherDevices.Visible = checkSearchROM.Checked;
         if (checkSearchROM.Checked)
         {
            if (adapter != null)
               refreshDeviceList(family_code);
         }
         else
         {
            comboDeviceList.Items.Clear();
            other_cnt = 0;
            selectedDevice = null;
         }
      }

      private bool SelectDevice()
      {
         OWResetResult rslt;

         try
         {
            // check if need to jump into overdrive
            if ((checkOverdrive.Checked) && (!inOverdrive))
            {
               statusWrite("// ___Device Select: put in Overdrive,  ",COMMENT_COLOR);

               // match ROM
               if (comboOverdrive.SelectedIndex == 0)
                  statusWrite("Overdrive Match ROM, FORMAT: 69h (command), change to overdrive, Registration Number (8 bytes)",COMMENT_COLOR);
               // skip ROM
               else if (comboOverdrive.SelectedIndex == 1)
                  statusWrite("Overdrive Skip-ROM, FORMAT: 3Ch (command), change to overdrive",COMMENT_COLOR);
               statusWrite("",RX_COLOR);

               statusWrite("<STD> R",TX_COLOR);
               adapter.Speed = OWSpeed.SPEED_REGULAR;
               rslt = adapter.Reset();
               if ((rslt == OWResetResult.RESET_PRESENCE) || (rslt == OWResetResult.RESET_ALARM))
               {
                  statusWrite("P ",RX_COLOR);
            
                  // ov match ROM
                  if (comboOverdrive.SelectedIndex == 0)
                  {
                     adapter.PutByte(0x69); // Match ROM cmd
                     statusWrite("69 <OVR> ",TX_COLOR);
                     adapter.Speed = OWSpeed.SPEED_OVERDRIVE;
                     adapter.DataBlock(selectedDevice, 0, 8);
                     statusWrite(ToHex(selectedDevice, 0, 8), TX_COLOR);
                     statusWrite("",RX_COLOR);
                     use_resume = true;
                     inOverdrive = true;
                  }                  
                  // ov skip ROM
                  else if (comboOverdrive.SelectedIndex == 1)
                  {
                     adapter.PutByte(0x3C); // Skip ROM cmd
                     statusWrite("3C <OVR> ",TX_COLOR);
                     adapter.Speed = OWSpeed.SPEED_OVERDRIVE;
                     statusWrite("",RX_COLOR);
                     use_resume = false;
                     inOverdrive = true;
                  }
                  return true;
               }
               else 
               {
                  statusWrite("N ",RX_COLOR);
                  statusWrite("",RX_COLOR);
                  use_resume = false;
                  inOverdrive = false;
                  return false;
               }
            }
            else
            {
               statusWrite("// ___Device Select: ",COMMENT_COLOR);
               // match ROM
               if (comboSelectMethod.SelectedIndex == 0)
               {
                  if (checkResume.Checked && use_resume)
                     statusWrite("Resume, FORMAT: A5h (command)",COMMENT_COLOR);
                  else      
                     statusWrite("Match ROM, FORMAT: 55h (command), Registration Number (8 bytes)",COMMENT_COLOR);
               }                  
               // skip ROM
               else if (comboSelectMethod.SelectedIndex == 1)
               {
                  statusWrite("Skip-ROM, FORMAT: CCh (command)",COMMENT_COLOR);
               }
               statusWrite("",RX_COLOR);

               // show current speed
               if (inOverdrive)
                  statusWrite("<OVR> ",TX_COLOR);
               else
               {
                  statusWrite("<STD> ",TX_COLOR);
                  adapter.Speed = OWSpeed.SPEED_REGULAR;
               }

               statusWrite("R",TX_COLOR);
               rslt = adapter.Reset();
               if ((rslt == OWResetResult.RESET_PRESENCE) || (rslt == OWResetResult.RESET_ALARM))
               {
                  statusWrite("P ",RX_COLOR);

                  // match ROM
                  if (comboSelectMethod.SelectedIndex == 0)
                  {
                     if (checkResume.Checked && use_resume)
                     {
                        adapter.PutByte(0xA5); // Match ROM cmd
                        statusWrite("A5 ",TX_COLOR);
                        statusWrite("",TX_COLOR);
                     }
                     else
                     {
                        adapter.PutByte(0x55); // Match ROM cmd
                        statusWrite("55 ",TX_COLOR);
                        adapter.DataBlock(selectedDevice, 0, 8);
                        statusWrite(ToHex(selectedDevice, 0, 8), TX_COLOR);
                        statusWrite("",RX_COLOR);
                        use_resume = true;
                     }
                  }                  
                     // skip ROM
                  else if (comboSelectMethod.SelectedIndex == 1)
                  {
                     adapter.PutByte(0xCC); // Skip ROM cmd
                     statusWrite("CC ",TX_COLOR);
                     statusWrite("",RX_COLOR);
                     use_resume = false;
                  }
                  return true;
               }
               else 
               {
                  statusWrite("N ",RX_COLOR);
                  statusWrite("",RX_COLOR);
                  use_resume = false;
                  inOverdrive = false;
                  return false;
               }
            }
         }
         catch(Exception ex)
         {
            statusWrite("<<" + ex.Message + ">>",ERROR_COLOR); 
            use_resume = false;
            inOverdrive = false;
            statusWrite("",ERROR_COLOR);
            return false;
         }
      }

      private bool ReadMem(int addr, int ln)
      {
         int tmp,prntlen;
         byte[] data = new byte[256];

         if (SelectDevice())
         {
            try
            {            
               statusWriteLN("// Read Memory, FORMAT: F0h (command), TA1 TA2 (address), read page data",COMMENT_COLOR); 
               // READ MEM cmd
               statusWrite("F0 ",TX_COLOR);
               adapter.PutByte(0xF0); 
               // TA1
               tmp = addr & 0xFF;
               statusWrite(tmp.ToString("X2") + " ",TX_COLOR);
               adapter.PutByte(tmp); 
               // TA2
               tmp = (addr >> 8) & 0xFF;
               statusWrite(tmp.ToString("X2") + " ",TX_COLOR);
               adapter.PutByte(tmp); 

               for (int i = 0; i < ln; i++)
               {
                  data[i] = (byte)adapter.GetByte();
                  statusWrite("[" + data[i].ToString("X2") + "] ",RX_COLOR);

                  // save value in global memory buffer
                  memimage[addr + i] = data[i];
               }

               statusWrite("",RX_COLOR);

               // Result
               statusWrite("// Address " + addr.ToString("X2") + "h,  length " + ln.ToString() + ", data read: ",COMMENT_COLOR);
               if (ln > 8)
               {
                  statusWrite("",COMMENT_COLOR);
                  for (int i = 0; i < ln; i+=8)
                  {
                     prntlen = ln - i;
                     if (prntlen > 8)
                        prntlen = 8;
                     statusWriteLN("//    " + toHexString(data,i,prntlen,32,true),COMMENT_COLOR);
                  }
               }
               else
               {
                  statusWrite(toHexString(data,0,ln,32,true),COMMENT_COLOR);
                  statusWrite("",COMMENT_COLOR);
               }

               return true;

            }
            catch(Exception ex)
            {
               statusWrite("<<" + ex.Message + ">>",ERROR_COLOR); 
               use_resume = false;
               inOverdrive = false;
               statusWrite("",ERROR_COLOR);
            }
         }
         return false;
      }

      private string toHexString(byte[] buffer, int offset, int count, int byteBreak, bool space)
      {
         StringBuilder sb = new StringBuilder(count*3 - 1);
         for(int i = 0; i<count; i++)
         {
            if(i!=0)
            {
               if (space)
                  sb.Append(" ");
               if(i%byteBreak==0)
                  sb.Append("\r\n");
            }
            sb.Append(buffer[offset+i].ToString("X2"));
         }
         return sb.ToString();
      }

      private void computePageMACButton_Click(object sender, System.EventArgs e)
      {
      }

      private void generateButton_Click(object sender, System.EventArgs e)
      {
      }

      public void do_crc8(int data)
      {
         crc8 = crc8 ^ data;
         for (int i = 0; i < 8; ++i)
         {
            if ((crc8 & 1) == 1)
               crc8 = (crc8 >> 1) ^ 0x8c;
            else
               crc8 = crc8 >> 1;
         }
      }

     
      private bool DS2431_write_scratchpad(int addr, int ln, byte[] data)
      {
         int tmp,i,rd;

         try
         {
            if (SelectDevice())
            {
               statusWriteLN("// Write Scratchpad, FORMAT: 0Fh (command), TA1 TA2 (address), data bytes, 2-byte CRC16",COMMENT_COLOR); 
               // write scratch cmd
               crc16 = 0;
               statusWrite("0F ",TX_COLOR);
               adapter.PutByte(0x0F); 
               do_crc16(0x0F);
               // TA1
               tmp = addr & 0xFF;
               statusWrite(tmp.ToString("X2") + " ",TX_COLOR);
               adapter.PutByte(tmp); 
               do_crc16(tmp);
               // TA2
               tmp = (addr >> 8) & 0xFF;
               statusWrite(tmp.ToString("X2") + " ",TX_COLOR);
               adapter.PutByte(tmp); 
               do_crc16(tmp);

               // scratchpad data
               for (i = 0; i < ln; i++)
               {
                  statusWrite(data[i].ToString("X2") + " ",TX_COLOR);
                  adapter.PutByte(data[i]);   
                  do_crc16(data[i]);
               }

               // check ending offsent
               if (((addr + ln - 1) & 0x07) == 0x07)
               {
                  // end of scratchpad so get crc16
                  for (i = 0; i < 2; i++)
                  {
                     tmp = (byte)adapter.GetByte();
                     statusWrite("[" + tmp.ToString("X2") + "] ",RX_COLOR);  
                     do_crc16(tmp);
                  }
                  statusWrite("",RX_COLOR);

                  if (crc16 == 0xB001)
                  {
                     statusWriteLN("// CRC16 verified to be correct", COMMENT_COLOR);
                     return true;
                  }
                  else
                  {
                     statusWriteLN("<<CRC16 incorrect, device may not be connected>>", ERROR_COLOR);
                     use_resume = false;
                     inOverdrive = false;
                     return false;
                  }
               }
               else
               {
                  // not end of scratchpad so do a read-scratchpad
                  statusWrite("",RX_COLOR);

                  if (SelectDevice())
                  {
                     statusWriteLN("// Read Scratchpad, FORMAT: AAh (command), TA1 TA2 (address), ES (offset),data bytes, 2-byte CRC16",COMMENT_COLOR); 
                     statusWrite("AA ",TX_COLOR);
                     adapter.PutByte(0xAA);

                     // read TA1
                     tmp = addr & 0xFF;
                     rd = adapter.GetByte();
                     statusWrite("[" + rd.ToString("X2") + "] ",RX_COLOR);  
                     if (tmp != rd)
                     {
                        statusWrite("",RX_COLOR);  
                        statusWriteLN("<< ERROR, TA1 address byte not correct >>", ERROR_COLOR);
                        return false;
                     }

                     // read TA2
                     tmp = (addr >> 8) & 0xFF;
                     rd = adapter.GetByte();
                     statusWrite("[" + rd.ToString("X2") + "] ",RX_COLOR);  
                     if (tmp != rd)
                     {
                        statusWrite("",RX_COLOR);  
                        statusWriteLN("<< ERROR, TA2 address byte not correct >>", ERROR_COLOR);
                        return false;
                     }

                     // read ES
                     rd = adapter.GetByte();
                     statusWrite("[" + rd.ToString("X2") + "] ",RX_COLOR);  
                     if (rd != ((ln + addr - 1) & 0x07))
                     {
                        statusWrite("",RX_COLOR);  
                        statusWriteLN("<< ERROR, ES byte not correct >>", ERROR_COLOR);
                        return false;
                     }

                     // read the data and compare
                     for (i = 0; i < ln; i++)
                     {
                        rd = adapter.GetByte();
                        statusWrite("[" + rd.ToString("X2") + "] ",RX_COLOR);  
                        if (rd != data[i])
                        {
                           statusWrite("",RX_COLOR);  
                           statusWriteLN("<< ERROR, data byte read back not correct >>", ERROR_COLOR);
                           return false;
                        }
                     }
                     statusWrite("",RX_COLOR);
                     return true;
                  }
               }
               return false;
            }
            else
               return false;
         }
         catch(Exception ex)
         {
            statusWriteLN("<< " + ex.Message + " >>",ERROR_COLOR);
            use_resume = false;
            inOverdrive = false;
            return false;
         }
      }

      private bool DS2431_write_block(byte addr, byte[] data, int len)
      {
         byte[] temp = new byte[128];
         byte[] mt = new byte[64];
         byte[] challenge = new byte[8];
         int i,tmp;

         try
         {
            // create buffer to write
            for (i = 0; i < 8; i++)
               temp[i] = memimage[i + (addr & 0xF8)];
            // overwrite new data
            for (i = 0; i < len; i++)
               temp[i + (addr & 0x07)] = data[i];

            statusWriteLN("// Put new data to write in the scratchpad",COMMENT_COLOR); 
            if (DS2431_write_scratchpad((addr & 0xF8),8,temp))
            {
               if (SelectDevice())
               {
                  // copy scratch command
                  statusWriteLN("// Copy Scratchpad, FORMAT: 55h (command), TA1 TA2 (address), ES (offset), delay for EE write, confirmation byte",COMMENT_COLOR); 
                  statusWrite("55 ",TX_COLOR);
                  adapter.PutByte(0x55);

                  // TA1
                  tmp = (addr & 0xF8) & 0xFF;
                  statusWrite(tmp.ToString("X2") + " ",TX_COLOR);
                  adapter.PutByte(tmp); 
                  // TA2
                  tmp = ((addr & 0xF8) >> 8) & 0xFF;
                  statusWrite(tmp.ToString("X2") + " ",TX_COLOR);
                  adapter.PutByte(tmp); 
                  // ES
                  statusWrite("07 <SP_ON> ",TX_COLOR);
                  adapter.StartPowerDelivery(OWPowerStart.CONDITION_AFTER_BYTE);
                  adapter.PutByte(0x07); 
                  statusWrite("",TX_COLOR);
            
                  // wait for EE write to complete
                  System.Threading.Thread.Sleep(10);
                  statusWriteLN("// Delay to allow for compute EEPROM write to complete, tPROG = 10ms (max)",COMMENT_COLOR);

                  // turn off strong pull-up
                  adapter.SetPowerNormal();
                  statusWrite("<SP_OFF> ",TX_COLOR);

                  // read toggle
                  tmp = (byte)adapter.GetByte();
                  statusWriteLN("[" + tmp.ToString("X2") + "] ",RX_COLOR);  
                  
                  if ((tmp == 0x55) || (tmp == 0xAA))
                  {
                     statusWriteLN("// Confirmation byte toggle valid ", COMMENT_COLOR);
                  }
                  else
                  {
                     statusWriteLN("<< ERROR, write-complete toggle not detected >>", ERROR_COLOR);
                     return false;
                  }

                  // update memory array for subsequent SHA computations
                  for (i = 0; i < len; i++)
                     memimage[addr + i] = data[i];

                  statusWriteLN("// Address " + addr.ToString("X2") + "h, length " + len.ToString() + ", data: " +
                     toHexString(data,0,len,32,true),COMMENT_COLOR);

                  return true;
               }
            }
            return false;
         }
         catch(Exception ex)
         {
            statusWriteLN("<< " + ex.Message + " >>",ERROR_COLOR);
            use_resume = false;
            inOverdrive = false;
            return false;
         }
      }



      private byte[] toByteArray(string strData)
      {
         string[] strBytes = strData.Split(' ', '\r', '\n');
         ArrayList byteList = new ArrayList(strBytes.Length);

         try
         {
            for(int i=0; i<strBytes.Length; i++)
               if(strBytes[i].Length>0)
                  byteList.Add((byte)int.Parse(strBytes[i], NumberStyles.HexNumber));
            byte[] bytes = new byte[byteList.Count];
            byteList.CopyTo(0, bytes, 0, bytes.Length);
            return bytes;
         }
         catch(Exception ex)
         {
            byte[] empty = new byte[0];
            return empty;
         }
      }

      private void anonymousComputePageMAC_Click(object sender, System.EventArgs e)
      {
      }

      private void loadFirstSecretButton_Click(object sender, System.EventArgs e)
      {
      }

      /*--------------------------------------------------------------------------
       * Calculate a new CRC16 from the input data integer.  Return the current
       * CRC16 and also update the global variable CRC16.
       */
      private int[] oddparity = new int[16] { 0, 1, 1, 0, 1, 0, 0, 1, 1, 0, 0, 1, 0, 1, 1, 0 };

      public void do_crc16(int data)
      {
         data = (data ^ (crc16 & 0xff)) & 0xff;
         crc16 >>= 8;

         if ((oddparity[data & 0xf] ^ oddparity[data >> 4]) > 0)
            crc16 ^= 0xc001;

         data <<= 6;
         crc16   ^= data;
         data <<= 1;
         crc16   ^= data;
      }

      private void writeButton_Click(object sender, System.EventArgs e)
      {
      }

      private void comboWriteAddress_SelectedIndexChanged(object sender, System.EventArgs e)
      {
      }

      private void computeNextSecretButton_Click(object sender, System.EventArgs e)
      {
      }


      public bool OWSearch(bool resetsearch, bool do_overdrive, byte cmd)
      {
         OWResetResult rslt;
         int id_bit_number;
         int last_zero, rom_byte_number;
         bool id_bit, cmp_id_bit, search_result;
         byte rom_byte_mask, search_direction;

         try
         {
            if (resetsearch)
            {
               // reset the search state
               LastDiscrepancy = 0;
               LastDeviceFlag = false;
               LastFamilyDiscrepancy = 0;
            }

            // initialize for search
            id_bit_number = 1;
            last_zero = 0;
            rom_byte_number = 0;
            rom_byte_mask = 1;
            search_result = false;
            crc8 = 0;

            // if the last call was not the last one
            if (!LastDeviceFlag)
            {       
               statusWriteLN("//------------------------------------------------------------------------------------", COMMENT_COLOR);
               statusWriteLN("// Device search, state reset = " + resetsearch.ToString() ,COMMENT_COLOR);

               if (cmd == 0xF0)
                  statusWriteLN("// Search-ROM, FORMAT: F0h (command), [read bit, read compliment, write direction bit] * 64",COMMENT_COLOR);
               else if (cmd == 0xEC)
                  statusWriteLN("// Conditional Search-ROM, FORMAT: ECh (command), [read bit, read compliment, write direction bit] * 64",COMMENT_COLOR);
               else
                  statusWriteLN("// Custom Search-ROM, FORMAT: " + cmd.ToString("X2") + "h (command), [read bit, read compliment, write direction bit] * 64",COMMENT_COLOR);

               // 1-Wire reset
               // show current speed
               if (do_overdrive)
                  statusWrite("<OVR> ",TX_COLOR);
               else
                  statusWrite("<STD> ",TX_COLOR);
                  
               statusWrite("R",TX_COLOR);
               rslt = adapter.Reset();
               if ((rslt != OWResetResult.RESET_PRESENCE) && (rslt == OWResetResult.RESET_ALARM))
               {
                  // reset the search
                  LastDiscrepancy = 0;
                  LastDeviceFlag = false;
                  LastFamilyDiscrepancy = 0;
                  return false;
               }
               else
                  statusWrite("P ",RX_COLOR);

               // issue the search command 
               adapter.PutByte(cmd);
               statusWrite(cmd.ToString("X2") + " ",TX_COLOR);
               statusWrite("",TX_COLOR);

               // loop to do the search
               do
               {
                  // read a bit and its complement
                  id_bit = adapter.GetBit();
                  if (id_bit)
                     statusWrite("[1] ",RX_COLOR);
                  else
                     statusWrite("[0] ",RX_COLOR);

                  cmp_id_bit = adapter.GetBit();
                  if (cmp_id_bit)
                     statusWrite("[1] ",RX_COLOR);
                  else
                     statusWrite("[0] ",RX_COLOR);

                  // check for no devices on 1-wire
                  if ((id_bit) && (cmp_id_bit))
                     break;
                  else
                  {
                     // all devices coupled have 0 or 1
                     if (id_bit != cmp_id_bit)
                     {
                        if (id_bit)
                           search_direction = 1;  // bit write value for search
                        else
                           search_direction = 0;
                     }
                     else
                     {
                        // if this discrepancy if before the Last Discrepancy
                        // on a previous next then pick the same as last time
                        if (id_bit_number < LastDiscrepancy)
                        {
                           if ((ROM_NO[rom_byte_number] & rom_byte_mask) > 0)
                              search_direction = 1;
                           else
                              search_direction = 0;
                        }
                        else
                        {
                           // if equal to last pick 1, if not then pick 0
                           if (id_bit_number == LastDiscrepancy)
                              search_direction = 1;
                           else
                              search_direction = 0;
                        }

                        // if 0 was picked then record its position in LastZero
                        if (search_direction == 0)
                        {
                           last_zero = id_bit_number;

                           // check for Last discrepancy in family
                           if (last_zero < 9)
                              LastFamilyDiscrepancy = last_zero;
                        }
                     }

                     // set or clear the bit in the ROM byte rom_byte_number
                     // with mask rom_byte_mask
                     if (search_direction == 1)
                        ROM_NO[rom_byte_number] |= rom_byte_mask;
                     else
                        ROM_NO[rom_byte_number] &= (byte)~rom_byte_mask;

                     // serial number search direction write bit
                     adapter.PutBit(search_direction==1);
                     statusWrite(search_direction.ToString() + " ",TX_COLOR);

                     // increment the byte counter id_bit_number
                     // and shift the mask rom_byte_mask
                     id_bit_number++;
                     rom_byte_mask <<= 1;

                     // if the mask is 0 then go to new SerialNum byte rom_byte_number and reset mask
                     if (rom_byte_mask == 0)
                     {
                        statusWrite("",COMMENT_COLOR);
                        statusWriteLN("// ROM BYTE #" + rom_byte_number.ToString() + " is " + ROM_NO[rom_byte_number].ToString("X2") + "h ",COMMENT_COLOR);

                        do_crc8(ROM_NO[rom_byte_number]);  // accumulate the CRC
                        rom_byte_number++;
                        rom_byte_mask = 1;
                     }
                  }
               }
               while(rom_byte_number < 8);  // loop until through all ROM bytes 0-7

               // if the search was successful then
               if (!((id_bit_number < 65) || (crc8 != 0)))
               {
                  // search successful so set LastDiscrepancy,LastDeviceFlag,search_result
                  LastDiscrepancy = last_zero;

                  // check for last device
                  if (LastDiscrepancy == 0)
                     LastDeviceFlag = true;
            
                  statusWriteLN("// CRC8 is correct, last discrepancy at bit " + LastDiscrepancy.ToString() + ", Last device flag = " + LastDeviceFlag.ToString(),COMMENT_COLOR);
                  statusWriteLN("// Device found: " + ToHex(ROM_NO, 0, 8),COMMENT_COLOR);

                  search_result = true;
               }
               else
               {
                  statusWrite("",COMMENT_COLOR);
                  statusWriteLN("// Search-ROM sequence did not find a device ",COMMENT_COLOR);
               }
            }
            else
               statusWriteLN("// End of Search ",COMMENT_COLOR);

            // if no device found then reset counters so next 'search' will be like a first
            if (!search_result || (ROM_NO[0] == 0))
            {
               LastDiscrepancy = 0;
               LastDeviceFlag = false;
               LastFamilyDiscrepancy = 0;
               search_result = false;
            }

            return search_result;
         }
         catch(Exception ex)
         {
            statusWrite("<<" + ex.Message + ">>",ERROR_COLOR); 
            use_resume = false;
            inOverdrive = false;
            statusWrite("",ERROR_COLOR);
            return false;
         }
      }

      private void refreshButton_Click(object sender, System.EventArgs e)
      {
         refreshDeviceList(family_code);
      }

      private void comboSelectMethod_SelectedIndexChanged(object sender, System.EventArgs e)
      {
         if (comboSelectMethod.SelectedIndex == 0)
         {
            // force check search ROM if match rom is selected
            if (!checkSearchROM.Checked)
               checkSearchROM.Checked = true;
         }
      }

      private void comboSpeed_SelectedIndexChanged(object sender, System.EventArgs e)
      {
         try
         {
            // show current speed
            if (comboSpeed.SelectedIndex == 0)
            {
               statusWrite("<STD> ",TX_COLOR);
               adapter.Speed = OWSpeed.SPEED_REGULAR;
            }
            else
            {
               statusWrite("<OVR> ",TX_COLOR);
               adapter.Speed = OWSpeed.SPEED_OVERDRIVE;
            }
         }
         catch(Exception ex)
         {
            statusWrite("<<" + ex.Message + ">>",ERROR_COLOR); 
            statusWrite("",ERROR_COLOR);
         }
      }

      private void readBitButton_Click(object sender, System.EventArgs e)
      {
         try
         {
            // read a bit and its complement
            bool rx_bit = adapter.GetBit();
            if (rx_bit)
               statusWrite("[1] ",RX_COLOR);
            else
               statusWrite("[0] ",RX_COLOR);

            if (afterbit)
            {
               statusWrite("<SP_ON> ",TX_COLOR);
               afterbit = false;
            }
         }
         catch(Exception ex)
         {
            statusWrite("<<" + ex.Message + ">>",ERROR_COLOR); 
            statusWrite("",ERROR_COLOR);
         }
      }

      private void strongPullupBitButton_Click(object sender, System.EventArgs e)
      {
         statusWrite("",COMMENT_COLOR);
         statusWriteLN("// Set power delivery after next bit", COMMENT_COLOR);    
     
         try
         {
            adapter.StartPowerDelivery(OWPowerStart.CONDITION_AFTER_BIT);
            afterbit = true;
         }
         catch(Exception ex)
         {
            statusWrite("<<" + ex.Message + ">>",ERROR_COLOR); 
            statusWrite("",ERROR_COLOR);
         }

      }

      private void button1_Click_1(object sender, System.EventArgs e)
      {
         byte[] cmd = FromHex(searchCommandText.Text);
         if (cmd.Length < 1)
         {
            MessageBox.Show("WARNING, There must be a Search Command in the search command text box." + 
               System.Environment.NewLine + "Typical choices are: F0 or EC.", AppTitle,
               MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
         }
         else
         {
            statusWrite("",COMMENT_COLOR);
            if (OWSearch(false,comboSpeed.SelectedIndex == 1,cmd[0]))
               textBoxROM.Text = ToHex(ROM_NO,0 ,8);
         }
      }

      private void deviceAddressList_SelectedIndexChanged(object sender, System.EventArgs e)
      {
      
      }

      private void menuItem34_Click(object sender, System.EventArgs e)
      {
         oneWireIOTextBox.Clear();
      }

      private void button1_Click_2(object sender, System.EventArgs e)
      {
         byte[] rom = new byte[8];
         try
         {
            statusWrite("",TX_COLOR);
            statusWrite("// Read ROM, FORMAT: 33h (command), read Registration Number (8 bytes)",COMMENT_COLOR);
            if (OWReset())
            {
               adapter.PutByte(0x33); // Read ROM cmd
               statusWrite("33 ",TX_COLOR);
               crc8 = 0;

               for (int i = 0; i < 8; i++)
               {
                  rom[i] = (byte)adapter.GetByte();
                  statusWrite("[" + rom[i].ToString("X2") + "] ",RX_COLOR);
                  do_crc8(rom[i]);  // accumulate the CRC
               }

               textBoxROM.Text = ToHex(rom,0,8);  
    
               statusWrite("",COMMENT_COLOR);
               if (crc8 == 0)
                  statusWriteLN("// CRC8 in ROM verified ",COMMENT_COLOR);
               else
                  statusWriteLN("<< ERROR, CRC8 in ROM is invalid, more then one 1-Wire device may be present>> ",ERROR_COLOR);

            }
         }
         catch(Exception ex)
         {
            statusWrite("<<" + ex.Message + ">>",ERROR_COLOR); 
            statusWrite("",ERROR_COLOR);
         }
      }

      private void button2_Click(object sender, System.EventArgs e)
      {
         try
         {
            statusWrite("",TX_COLOR);
            statusWrite("// Resume Command, FORMAT: A5h (command)",COMMENT_COLOR);
            if (OWReset())
            {
               adapter.PutByte(0xA5); // Skip ROM cmd
               statusWrite("A5 ",TX_COLOR);
            }
         }
         catch(Exception ex)
         {
            statusWrite("<<" + ex.Message + ">>",ERROR_COLOR); 
            statusWrite("",ERROR_COLOR);
         }       
      }

      private void menuHelp_Click(object sender, System.EventArgs e)
      {
         MessageBox.Show(AppTitle + System.Environment.NewLine + AppVersion,AppTitle);
      }

      private void comboNextPage_SelectedIndexChanged(object sender, System.EventArgs e)
      {
      }

      private void refreshSPButton_Click(object sender, System.EventArgs e)
      {
      }

      private void OneWireIO_Load(object sender, System.EventArgs e)
      {
      
      }

      private void OneWireIO_Resize(object sender, System.EventArgs e)
      {
         tabControl.Size = new Size(ClientSize.Width - 16, ClientSize.Height - 24);

         AdapterGroup.Size = new Size(tabControl.ClientRectangle.Width - 24, AdapterGroup.Size.Height);
         DeviceGroup.Size = new Size(tabControl.ClientRectangle.Width - 24, DeviceGroup.Size.Height);
         lowLevelBox.Size = new Size(tabControl.ClientRectangle.Width - 24, lowLevelBox.Size.Height);
         ROMGroup.Size = new Size(tabControl.ClientRectangle.Width - 24, ROMGroup.Size.Height);

      }

      private void textSecret_DoubleClick(object sender, System.EventArgs e)
      {
      }

      private void textChallenge_DoubleClick(object sender, System.EventArgs e)
      {
      }

      private void Pg3InputBox_DoubleClick(object sender, System.EventArgs e)
      {
      }

      private void textNewData_DoubleClick(object sender, System.EventArgs e)
      {
      }

      private void textWriteSecret_DoubleClick(object sender, System.EventArgs e)
      {
      }

      private void textNextSecret_TextChanged(object sender, System.EventArgs e)
      {
      }

      private void textPartialSecret_DoubleClick(object sender, System.EventArgs e)
      {
      }

      private void textNewSecret_DoubleClick(object sender, System.EventArgs e)
      {
      }

      private void textNextSecret_DoubleClick(object sender, System.EventArgs e)
      {
      }

      private void bytesTextBox_DoubleClick(object sender, System.EventArgs e)
      {
      }

      private void textBoxROM_DoubleClick(object sender, System.EventArgs e)
      {
         textBoxROM.SelectionLength = 2;
      }

      private void bytesTextBox_Click(object sender, System.EventArgs e)
      {
         if (bytesTextBox.SelectionLength == 3)
            bytesTextBox.SelectionLength = 2;
      }

      private void button3_Click(object sender, EventArgs e)
      {
         statusWrite("", COMMENT_COLOR);
         statusWriteLN("// 7V Pulse", COMMENT_COLOR);

         try
         {
            adapter.Start7VPulse(OWPowerStart.CONDITION_NOW);
         }
         catch (Exception ex)
         {
            statusWrite("<<" + ex.Message + ">>", ERROR_COLOR);
            statusWrite("", ERROR_COLOR);
         }
      }

      private void button4_Click(object sender, EventArgs e)
      {
         statusWrite("", COMMENT_COLOR);
         statusWriteLN("// POR, 1 second", COMMENT_COLOR);

         try
         {
            adapter.StartBreak();
            System.Threading.Thread.Sleep(1000);
            adapter.SetPowerNormal();
         }
         catch (Exception ex)
         {
            statusWrite("<<" + ex.Message + ">>", ERROR_COLOR);
            statusWrite("", ERROR_COLOR);
         }

      }

      private void menuItemRunScript_Click(object sender, EventArgs e)
      {
         bool error_detected = false;
         int number_of_files = 0;

         OpenFileDialog fileOpen = new OpenFileDialog();
         fileOpen.InitialDirectory = ".\\";
         fileOpen.Filter = "Text files (*.txt)|*.txt|All files (*.*)|*.*";
         fileOpen.FilterIndex = 0;
         fileOpen.RestoreDirectory = false; //true;
         fileOpen.Multiselect = true;

         if (fileOpen.ShowDialog() != DialogResult.OK)
         {
            return;
         }
         
         // Read the files
         foreach (String file in fileOpen.FileNames)
         {
            number_of_files++;

            if (RunScript(file))
               error_detected = true;
         }

         if (number_of_files > 1)
         {
            if (error_detected)
               MessageBox.Show("WARNING, Error detected in the results of one of the scripts.", AppTitle,
                  MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
            else
               MessageBox.Show("No erros detected.", AppTitle,
                  MessageBoxButtons.OK, MessageBoxIcon.Information);
         }
      }

      private bool RunScript(String file)
      {
         OWResetResult rslt;
         string sb;
         char[] c, n;
         bool id_bit, delay_done, sp_on;
         int data, delay;
         bool error_detected = false;

         string[] lines = System.IO.File.ReadAllLines(file); //?????

         statusWriteLN("", COMMENT_COLOR);
         // String timeStamp = GetTimestamp(DateTime.Now);
         statusWriteLN("//------------------------------------------------------------------------------------", COMMENT_COLOR);
         statusWriteLN("// Running Script (" + DateTime.Now + "): " + file, COMMENT_COLOR);
         statusWriteLN("//------------------------------------------------------------------------------------", COMMENT_COLOR);

         sp_on = false;
         delay_done = false;
         mask_scroll = true;

         foreach (string line in lines)
         {
            try
            {
               // Parse the line
               if (line.StartsWith("//"))
               {
                  // comment
                  statusWriteLN(line, COMMENT_COLOR);
               }
               else
               {
                  // Instead of looping on string lengt, call function to tokenize the line, 
                  // make a seperate string for each non-whitepace string in the line 
                  string[] tokens = line.Split(' ');
                  for (int t = 0; t < tokens.GetLength(0); t++)
                  {
                     // check for dummy token
                     if (tokens[t].Length == 0)
                        continue;

                     // convert token to array of characters
                     c = tokens[t].ToCharArray();

                     if (c[0] == 'R')
                     {
                        // 1-wire reset
                        statusWrite("R", TX_COLOR);
                        rslt = adapter.Reset();
                        if ((rslt == OWResetResult.RESET_PRESENCE) || (rslt == OWResetResult.RESET_ALARM))
                           statusWrite("P ", RX_COLOR);
                        else
                           statusWrite("N ", RX_COLOR);
                     }
                     else if (c[0] == '<')
                     {
                        // special command 
                        if (tokens[t].StartsWith("<STD>"))
                        {
                           // check if already STD
                           if (comboSpeed.SelectedIndex == -1)
                              comboSpeed.SelectedIndex = 0;
                           else if (comboSpeed.SelectedIndex == 0)
                              statusWrite("<STD> ", TX_COLOR);
                           else
                              comboSpeed.SelectedIndex = 0;
                        }
                        else if (tokens[t].StartsWith("<OVR>"))
                        {
                           // check if already OVR
                           if (comboSpeed.SelectedIndex == -1)
                              comboSpeed.SelectedIndex = 1;
                           else if (comboSpeed.SelectedIndex == 1)
                              statusWrite("<OVR> ", TX_COLOR);
                           else
                              comboSpeed.SelectedIndex = 1;
                        }
                        else if (tokens[t].StartsWith("<SP_ON>"))
                        {
                           // skip turning it on if it is already on
                           if (!sp_on)
                              adapter.StartPowerDelivery(OWPowerStart.CONDITION_NOW);
                           statusWrite("<SP_ON> ", TX_COLOR);
                        }
                        else if (tokens[t].StartsWith("<SP_OFF>"))
                        {
                           // check if detected a delay token
                           if (!delay_done)
                           {
                              statusWrite("<DELAY200> ", TX_COLOR);
                              System.Threading.Thread.Sleep(200);
                           }

                           adapter.SetPowerNormal();
                           statusWrite("<SP_OFF> ", TX_COLOR);

                           delay_done = false;
                        }
                        else if (tokens[t].StartsWith("<7VPULSE>"))
                        {
                           // do 7v pulse on VCC
                           adapter.Start7VPulse(OWPowerStart.CONDITION_NOW);

                           statusWrite("<7VPULSE> ", TX_COLOR);
                        }
                        else if (tokens[t].StartsWith("<12VPULSE>"))
                        {
                           // do 12v pulse on 1-Wire
                           adapter.StartProgramPulse(OWPowerStart.CONDITION_NOW);

                           statusWrite("<12VPULSE> ", TX_COLOR);
                        }
                        else if (tokens[t].StartsWith("<PAUSE>"))
                        {
                           statusWrite("<PAUSE> ", TX_COLOR);
                           MessageBox.Show("Pause detected in script, click OK to continue", AppTitle,
                              MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                        }
                        else if (tokens[t].StartsWith("<POWERDOWN>"))
                        {
                           adapter.StartBreak();
                           statusWrite("<POWERDOWN> ", TX_COLOR);
                        }
                        else if (tokens[t].StartsWith("<DELAY"))
                        {
                           // Delay 

                           // get substring after delay token 
                           sb = tokens[t].Substring(6, tokens[t].Length - 6);

                           // make sure that we have at least 2 characters
                           if (sb.Length < 2)
                              continue;

                           // convert to characters
                           n = sb.ToCharArray();

                           // loop until find '>' and build up a decimal number
                           delay = 0;
                           for (int j = 0; j < sb.Length; j++)
                           {
                              if (n[j] == '>')
                                 break;
                              else if ((n[j] >= '0') && (n[j] <= '9'))
                              {
                                 delay *= 10;
                                 delay += (n[j] - '0');
                              }
                           }

                           statusWrite("<DELAY" + delay.ToString() + "> ", TX_COLOR);
                           System.Threading.Thread.Sleep(delay);

                           delay_done = true;
                        }
                        else if (tokens[t].StartsWith("<<"))
                        {
                           statusWrite("// " + line, COMMENT_COLOR);
                           break;
                        }
                     }
                     // check for read
                     else if (c[0] == '[')
                     {
                        if (tokens[t].Length == 3)
                        {
                           // check for next token is strong pull-up
                           if ((t + 1) < tokens.GetLength(0))
                           {
                              // at least one more token
                              if (tokens[t + 1].StartsWith("<SP_ON>"))
                              {
                                 adapter.StartPowerDelivery(OWPowerStart.CONDITION_AFTER_BIT);
                                 //???statusWrite("<<SP PRIMED FOR NEXT BIT>> ", ERROR_COLOR);

                                 sp_on = true;
                              }
                              else
                                 sp_on = false;
                           }

                           // read bit
                           id_bit = adapter.GetBit();

                           // check for [??] format
                           if (c[1] == '?')
                           {
                              if (id_bit)
                                 statusWrite("[1] ", RX_COLOR);
                              else
                                 statusWrite("[0] ", RX_COLOR);
                           }
                           else
                           {
                              if ((id_bit && c[1] == '1') || (!id_bit && c[1] == '0'))
                              {
                                 if (id_bit)
                                    statusWrite("[1] ", RX_COLOR);
                                 else
                                    statusWrite("[0] ", RX_COLOR);
                              }
                              else
                              {
                                 if (id_bit)
                                    statusWrite("[1] ", ERROR_COLOR);
                                 else
                                    statusWrite("[0] ", ERROR_COLOR);
                                 error_detected = true;
                              }
                           }
                        }
                        else if (tokens[t].Length == 4)
                        {
                           // check for next token is strong pull-up
                           if ((t + 1) < tokens.GetLength(0))
                           {
                              // at least one more token
                              if (tokens[t + 1].StartsWith("<SP_ON>"))
                              {
                                 adapter.StartPowerDelivery(OWPowerStart.CONDITION_AFTER_BYTE);
                                 //???statusWrite("<<SP PRIMED FOR NEXT BYTE>> ", ERROR_COLOR);

                                 sp_on = true;
                              }
                              else
                                 sp_on = false;
                           }

                           // read byte
                           data = adapter.GetByte();

                           // check for [??] format
                           if (c[1] == '?')
                           {
                              // don't compare
                              statusWrite("[" + data.ToString("X2") + "] ", RX_COLOR);
                           }
                           else
                           {
                              // check if different then log read
                              byte[] rx = FromHex(tokens[t]);

                              if (data != (int)rx[0])
                              {
                                 statusWrite("[" + data.ToString("X2") + "] ", ERROR_COLOR);
                                 error_detected = true;
                              }
                              else
                                 statusWrite("[" + data.ToString("X2") + "] ", RX_COLOR);
                           }
                        }
                     }
                     else if (((c[0] >= '0') && (c[0] <= '9')) || ((c[0] >= 'A') && (c[0] <= 'F')))
                     {
                        // write 

                        if (tokens[t].Length == 1)
                        {
                           // check for next token is strong pull-up
                           if ((t + 1) < tokens.GetLength(0))
                           {
                              // at least one more token
                              if (tokens[t + 1].StartsWith("<SP_ON>"))
                              {
                                 adapter.StartPowerDelivery(OWPowerStart.CONDITION_AFTER_BIT);
                                 //???statusWrite("<<SP PRIMED FOR NEXT BIT>> ", ERROR_COLOR);

                                 sp_on = true;
                              }
                              else
                                 sp_on = false;
                           }

                           // write bit
                           if (c[0] == '0')
                           {
                              adapter.PutBit(false);
                              statusWrite("0 ", TX_COLOR);
                           }
                           else if (c[0] == '1')
                           {
                              adapter.PutBit(true);
                              statusWrite("1 ", TX_COLOR);
                           }
                        }
                        else if (tokens[t].Length == 2)
                        {
                           // check for next token is strong pull-up
                           if ((t + 1) < tokens.GetLength(0))
                           {
                              // at least one more token
                              if (tokens[t + 1].StartsWith("<SP_ON>"))
                              {
                                 adapter.StartPowerDelivery(OWPowerStart.CONDITION_AFTER_BYTE);
                                 //???statusWrite("<<SP PRIMED FOR NEXT BYTE>> ", ERROR_COLOR);

                                 sp_on = true;
                              }
                              else
                                 sp_on = false;
                           }

                           // write byte
                           byte[] tx = FromHex(tokens[t]);
                           adapter.PutByte(tx[0]);
                           statusWrite(tx[0].ToString("X2") + " ", TX_COLOR);
                        }

                     }
                  }
                  statusWriteLN("", COMMENT_COLOR);
               }
            }
            catch (Exception ex)
            {
               statusWrite("// ERROR: " + line, ERROR_COLOR);
               error_detected = true;
            }
         }

         mask_scroll = false;

         // check if we are in hide log mode, temporarily enable so we can see result
         bool temporary_enable_log = hide_log_mode;
         if (temporary_enable_log)
         {
            hide_log_mode = false;
         }

         if (error_detected)
            statusWriteLN("// Script Result: FAIL", COMMENT_COLOR);
         else
            statusWriteLN("// Script Result: SUCCESS", COMMENT_COLOR);

         // if in hide long mode, re-enable
         if (temporary_enable_log)
         {
            hide_log_mode = true;
         }

         statusWriteLN("//------------------------------------------------------------------------------------", COMMENT_COLOR);
         statusWriteLN("// End Script: " + file, COMMENT_COLOR);
         statusWriteLN("//------------------------------------------------------------------------------------", COMMENT_COLOR);

         LastScriptName = file; // remember last script
         buttonRepeatScript.Enabled = true; // enable repeat button

         return  error_detected;
      }


      private void menuItemSaveScript_Click(object sender, EventArgs e)
      {
         SaveFileDialog fileOpen = new SaveFileDialog();
         fileOpen.InitialDirectory = ".\\";
         fileOpen.Filter = "Text files (*.txt)|*.txt|All files (*.*)|*.*";
         fileOpen.FilterIndex = 0;
         fileOpen.RestoreDirectory = false; //true;
         //                if (fileOpen.ShowDialog () == DialogResult.Cancel)
         if (fileOpen.ShowDialog() != DialogResult.OK)
         {
            return;
         }

         // save log
         System.IO.File.WriteAllText(fileOpen.FileName, oneWireIOTextBox.Text);

      }

      private void buttonRepeatScript_Click(object sender, EventArgs e)
      {
         for (int i = 1; i <= repeatScriptCount.Value; i++)
         {
            RunScript(LastScriptName);
            if (repeatScriptCount.Value > 1)
               statusWriteLN("//-- Repeat Count: " + i.ToString() + " of " + repeatScriptCount.Value.ToString() + " ------------------------------------", COMMENT_COLOR);
         }
      }

      private void checkBoxHideLog_CheckedChanged(object sender, EventArgs e)
      {
         hide_log_mode = checkBoxHideLog.Checked;
      }

      private void button12VPulse_Click(object sender, EventArgs e)
      {
         if (MessageBox.Show("WARNING, The following operation will put a 12V 512us pulse on the 1-Wire. Any non EPROM will be damaged!" +
            System.Environment.NewLine + "CANCEL to abort operation or OK to continue.", AppTitle,
            MessageBoxButtons.OKCancel, MessageBoxIcon.Exclamation) == DialogResult.Cancel)
         {
            statusWriteLN("// Operation Aborted", COMMENT_COLOR);
            return;
         }

         try
         {
            if (adapter.StartProgramPulse(OWPowerStart.CONDITION_NOW))
               statusWrite("<12VPULSE> ", TX_COLOR);
            else
               statusWrite("<< ERROR, pulse did not complete >>", ERROR_COLOR);

         }
         catch (Exception ex)
         {
            statusWrite("<<" + ex.Message + ">>", ERROR_COLOR);
            statusWrite("", ERROR_COLOR);
         }

      }
   }
}
